#pragma warning(disable:4786)
#include "quaternion.h"

//=======================================================================================
// camera.cpp  | Feb 2004
//---------------------------------------------------------------------------------------
//
//	Camera class for openGL,	[03.11.2003]
//
//	based loosely on ultimategame programming.com tutorials.  well, right now the
//	quaternion code is based exactly on this.
//	as a side, these tutorials are pretty good, although the writer seems to fancy 
//	himself as a bit of a twat.  or programming guru to the kings.  but more like a twat.
//
//---------------------------------------------------------------------------------------
//  upd.						[23.02.2004]
//
//  Tracking functions implemented.
//  Uses springs for nicer, smoother corners and chasing.
//
//=======================================================================================

#include "camera.h"

//--------------------------------------------------------------------------------
// build the camera and give everything some nice values to start with
//--------------------------------------------------------------------------------
CCamera::CCamera(){
	//basic setup
	m_Position.Set	(0,0,0);
	m_Lookat.Set	(0,0,1);
	m_Up.Set		(0,1,0);
	m_Slide.Set		(0,0,0);
	m_Velocity.Set	(0,0,0);

	m_CurrentAngle	=	0.0f;

	//spring setup
	m_Bolt.Set(0, 15.0, -80);
	m_TheSpring.Set(0.9f,5.0f);
	m_SpringDeltaT	=	1.0f;
	m_SpringFriction	=	0.29f;
	m_SpringForce.Set(0,0,0);
}

//--------------------------------------------------------------------------------
// This builds a view projection matrix[well for the rotation] from the 
// position/lookat/up. 
//--------------------------------------------------------------------------------
void CCamera::BuildMatrix()
{
	vec3f L, S, U;

	L=m_Lookat-m_Position;
	L.Normalize();

	S=L.CrossProduct(m_Up);
	S.Normalize();

	U=S.CrossProduct(L);
	U.Normalize();
	
	m_RotationMatrix.Set(0, S.x, U.x, -L.x, 0);
	m_RotationMatrix.Set(1, S.y, U.y, -L.y, 0);
	m_RotationMatrix.Set(2, S.z, U.z, -L.z, 0);
	m_RotationMatrix.Set(3, 0, 0, 0, 1);

	m_ViewMatrix.Set(0, 1, 0, 0, 0);
	m_ViewMatrix.Set(1, 0, 1, 0, 0);
	m_ViewMatrix.Set(2, 0, 0, 1, 0);
	m_ViewMatrix.Set(3, -m_Position.x, -m_Position.y, -m_Position.z, 1);

	m_ViewMatrix = m_ViewMatrix * m_RotationMatrix;
}

//--------------------------------------------------------------------------------
// Get's whatever matrix you ask for
//--------------------------------------------------------------------------------
void CCamera::GetMatrix(camRotationMatrix mtx, mat4f &matrix)
{
	BuildMatrix();

	switch (mtx)
	{
	case MATRIX_MODELVIEW:
		matrix = m_ViewMatrix;
		return;
	case MATRIX_PROJECTION:
		matrix = m_ProjectionMatrix;
		return;
	case MATRIX_ROTATION:
		matrix = m_RotationMatrix;
		return;
	}
}

mat4f CCamera::GetMatrix(camRotationMatrix mtx)
{
	BuildMatrix();

	switch (mtx)
	{
	case MATRIX_MODELVIEW:
		return m_ViewMatrix;		
	case MATRIX_PROJECTION:
		return m_ProjectionMatrix;		
	case MATRIX_ROTATION:
		return m_RotationMatrix;		
	}

	return m_ViewMatrix;
}

//--------------------------------------------------------------------------------
// Returns the last calculated matrix
//--------------------------------------------------------------------------------
void CCamera::GetMatrixNoBuild(camRotationMatrix mtx, mat4f &matrix)
{
	switch (mtx)
	{
	case MATRIX_MODELVIEW:
		matrix = m_ViewMatrix;
		return;
	case MATRIX_PROJECTION:
		matrix = m_ProjectionMatrix;
		return;
	case MATRIX_ROTATION:
		matrix = m_RotationMatrix;
		return;
	}
}

mat4f CCamera::GetMatrixNoBuild(camRotationMatrix mtx)
{

	switch (mtx)
	{
	case MATRIX_MODELVIEW:
		return m_ViewMatrix;		
	case MATRIX_PROJECTION:
		return m_ProjectionMatrix;		
	case MATRIX_ROTATION:
		return m_RotationMatrix;		
	}

	return m_ViewMatrix;

}

//--------------------------------------------------------------------------------
// Call this at the very start to initialise the camera
//--------------------------------------------------------------------------------
void CCamera::SetCamera(float x, float y, float z,
					   float vx, float vy, float vz,
					   float ux, float uy, float uz)
{	
	m_Position.x = x;
	m_Position.y = y;
	m_Position.z = z;
	
	m_Lookat.x = vx;
	m_Lookat.y = vy;
	m_Lookat.z = vz;
	
	m_Up.x = ux;
	m_Up.y = uy;
	m_Up.z = uz;
	
}

//--------------------------------------------------------------------------------
// Creates a View Projection Matrix
//--------------------------------------------------------------------------------
void CCamera::SetProjection(float _fov, float _aspectRatio, float _zNear, float _zFar)
{
	m_FOV	= _fov;
	m_Aspect= _aspectRatio;
	m_ZNear = _zNear;
	m_ZFar	= _zFar;

	float f, a, b, c;

	f = float(1/tan((_fov/2)*PIOVER180));
	a = f / _aspectRatio;
	b = (_zFar + _zNear) / (_zNear - _zFar);
	c = (2 * _zFar * _zNear) / (_zNear - _zFar); 

	m_ProjectionMatrix.Set(0, a, 0, 0, 0);
	m_ProjectionMatrix.Set(1, 0, f, 0, 0);
	m_ProjectionMatrix.Set(2, 0, 0, b, -1);
	m_ProjectionMatrix.Set(3, 0, 0, c, 0);

}
//--------------------------------------------------------------------------------
// Set some variables
//--------------------------------------------------------------------------------
bool CCamera::Set(camVariable _me, vec3f &_tothis)
{
	switch(_me)
	{
	case CAMERA_POSITION:
		m_Position = _tothis;
		return TRUE;
	case CAMERA_LOOKAT:
		m_Lookat = _tothis;
		return TRUE;
	case CAMERA_VELOCITY:
		m_Velocity = _tothis;
		return TRUE;
	case CAMERA_UP:
		m_Up = _tothis;
		return TRUE;
	case CAMERA_BOLT:
		m_Bolt = _tothis;
		return TRUE;
	}
	return FALSE;
}

bool CCamera::Change(camVariable _me, vec3f &_amount)
{
	switch(_me)
	{
	case CAMERA_POSITION:
		m_Position += _amount;
		return TRUE;
	case CAMERA_LOOKAT:
		m_Lookat += _amount;
		return TRUE;
	case CAMERA_VELOCITY:
		m_Velocity += _amount;
		return TRUE;
	case CAMERA_UP:
		m_Up += _amount;
		return TRUE;
	case CAMERA_BOLT:
		m_Bolt += _amount;
		return TRUE;
	}
	return FALSE;
}

//--------------------------------------------------------------------------------
// change the zoom of the camera [field of view]
// MUST BE FOLLOWED BY A SET PROJECTION() FROM THE RENDERER TO HAVE EFFECT
//--------------------------------------------------------------------------------
bool CCamera::Set(camVariable _me, float _tothis)
{
	switch(_me)
	{
	case CAMERA_VELOCITY_MAX:
		m_Vmax = _tothis;
		return TRUE;
	case CAMERA_ZPLANE_FAR:
		m_ZFar = _tothis;
		SetProjection(m_FOV, m_Aspect, m_ZNear, m_ZFar);
		return TRUE;
	case CAMERA_ZPLANE_NEAR:
		m_ZNear = _tothis;
		SetProjection(m_FOV, m_Aspect, m_ZNear, m_ZFar);
		return TRUE;
	case CAMERA_ZOOM:
		m_FOV = _tothis;
		SetProjection(m_FOV, m_Aspect, m_ZNear, m_ZFar);
		return TRUE;	
	case CAMERA_TWIST:
		m_Up.Set(0.0f,1.0f,0.0f);
		m_Up.RotateAround((m_Lookat - m_Position), _tothis);
		return TRUE;

	}
	return FALSE;
}

bool CCamera::Change(camVariable _me, float _amount)
{
	switch(_me)
	{
	case CAMERA_VELOCITY_MAX:
		m_Vmax += _amount;
		return TRUE;
	case CAMERA_ZPLANE_FAR:
		m_ZFar += _amount;
		SetProjection(m_FOV, m_Aspect, m_ZNear, m_ZFar);
		return TRUE;
	case CAMERA_ZPLANE_NEAR:
		m_ZNear += _amount;
		SetProjection(m_FOV, m_Aspect, m_ZNear, m_ZFar);
		return TRUE;
	case CAMERA_ZOOM:
		m_FOV += _amount;
		SetProjection(m_FOV, m_Aspect, m_ZNear, m_ZFar);
		return TRUE;
	case CAMERA_TWIST:
		m_Up.RotateAround((m_Lookat - m_Position), _amount);
		return TRUE;

	}
	return FALSE;
}

//--------------------------------------------------------------------------------
// change velocity by amounts, [!= setvelocity]
//--------------------------------------------------------------------------------
void CCamera::ModifyVelocity(float plusx, float plusy, float plusz)
{
	m_Velocity.x += plusx;
	m_Velocity.y += plusy;
	m_Velocity.z += plusz;
}

//--------------------------------------------------------------------------------
// makes for nicer, more natural movement, as opposed to the jerky jumping when 
// adv/strafe/ascend are called seperately
//--------------------------------------------------------------------------------
void CCamera::UpdateUsingVelocity(float timelapsed, float friction)
{
	//x = strafe
	//z = advance
	//y = Ascend

	Advance	((m_Velocity.z *= friction) * timelapsed);
	Strafe	((m_Velocity.x *= friction) * timelapsed);
	Ascend	((m_Velocity.y *= friction) * timelapsed);		
}

//--------------------------------------------------------------------------------
// general update function, used by following ones for niceness, 
// inlineing might be better
//--------------------------------------------------------------------------------
void CCamera::Update(vec3f direction, float amount)
{
	m_Position += direction * amount;
	m_Lookat += direction * amount;
}

//--------------------------------------------------------------------------------
// forwards / back
//--------------------------------------------------------------------------------
void CCamera::Advance(float amount)
{
	vec3f VecFacing = m_Lookat - m_Position;	//get direction we're facing in
	VecFacing.Normalize();

	Update(VecFacing, amount);
}

//--------------------------------------------------------------------------------
// move along relative 'up' direction
//--------------------------------------------------------------------------------
void CCamera::Ascend(float amount)
{
	Update(m_Up, amount);
}

//--------------------------------------------------------------------------------
// side to side
//--------------------------------------------------------------------------------
void CCamera::Strafe(float amount)
{
	vec3f VecFacing, VecStrafe;

	//get side to side vector so we can strafe along it.
	VecFacing = m_Lookat - m_Position;
	VecFacing.Normalize();
	VecStrafe = VecFacing.CrossProduct(m_Up);

	Update(VecStrafe, amount);

}

//--------------------------------------------------------------------------------
// aaaargh! quaternions!
//--------------------------------------------------------------------------------
void CCamera::Rotate(float AngleDir, float amountx, float amounty, float amountz)
{
	quaternion qRotation, qView, qNewView;
	
	// Create the rotation quaternion based on the axis we are rotating on.
	qRotation.Rotatef(AngleDir, amountx, amounty, amountz);
	
	// Create the view quaternion.  This will be the direction of the view and m_position.
	qView.x = m_Lookat.x - m_Position.x;
	qView.y = m_Lookat.y - m_Position.y;
	qView.z = m_Lookat.z - m_Position.z;
	qView.w = 0;
	
	// Create the resulting quaternion by multiplying the rotation quat by the view quat
	// then multiplying that by the conjugate of the rotation quat.
	qNewView = ((qRotation * qView) * qRotation.Conjugate());
	
	// m_update the view information by adding the m_position to the resulting quaternion.
	m_Lookat.x = m_Position.x + qNewView.x;
	m_Lookat.y = m_Position.y + qNewView.y;
	m_Lookat.z = m_Position.z + qNewView.z;
}


//--------------------------------------------------------------------------------
// tmouse is a struct, will most likely be improved upon and this function will 
// be marginally different, hurrah!
//--------------------------------------------------------------------------------
void CCamera::RotateYX(float y, float x, float scale)
{
	float xDirection = 0.0f;         // Direction angle.
	float yRotation = 0.0f;          // Rotation angle.

	// x and y rotation amounts
	xDirection	= -x * scale;		
	yRotation	= y * scale;		

	// rotate based on where we are facing right now
	m_CurrentAngle -= yRotation;

   // get side to side vector(x) for pitch rotation
   vec3f VecAxis, VecFacing;
   //VecAxis = 0.0f;
   //VecFacing = 0.0f;

   // Get the Direction of the view.
   VecFacing = m_Lookat - m_Position;

   // Get the cross product of the direction and the up.
   VecAxis = VecFacing.CrossProduct(m_Up);

   // Normalize it.
	VecAxis.Normalize();	

   // Rotate the camera.
   Rotate(yRotation, VecAxis.x, VecAxis.y, VecAxis.z);
   //Rotate(xDirection, m_Up.x, m_Up.y, m_Up.z);

   Rotate(xDirection, 0, 1, 0);	
   //use this if you want more 'pleasant' rotation, ie always around y.
   //altough of course, right now m_Up never changes, so its a bit of a waste of time really.

}

//--------------------------------------------------------------------------------
// Stick camera exactly m_Bolt from object, based on direction object is facing
// BoltHere - vector FROM object
// LookHere - where to look, usually object
//--------------------------------------------------------------------------------
void CCamera::Track_BoltOn(vec3f &Position, vec3f &Velocity, vec3f &LookHere)
{
	m_Position = Position;
	m_Lookat   = Position+Velocity;

	Advance	(m_Bolt.z);
	Strafe	(m_Bolt.x);
	Ascend	(m_Bolt.y);

	m_Lookat = LookHere;
}


//================================================================================
// Spring based camera functions, setting up and updates.
//--------------------------------------------------------------------------------
//
// The constructor fills everything up with what i reckon are decent values to 
// show whats going on, chances are they're not the best for the situation.
// I just dont like lots of NULL's that crash me without warning
//
//================================================================================

//k=0.8 might be decent - 
//k = springiness, deltaT = restLength length = start length, friction  = friction.
void CCamera::SetSpring(float k_coefficient, float length, float friction, float deltaT)

{
	m_TheSpring.Set(k_coefficient, length);
	m_SpringDeltaT		=	deltaT;
	m_SpringFriction	=	friction;

}


//---------------------------------------------------------------- SPRING AND BOLT
//Smoother camera, moves to behind the object.
//Because this method uses a bolt, velocity and more is required
//BoltHere - vector FROM object
//LookHere - where to look, usually object
//--------------------------------------------------------------------------------
void CCamera::SpringDampenedBolt(vec3f &Position, vec3f &Velocity, vec3f &LookHere)
{

	//Calculate bolt position in super space of camera, based on relative position BoltHere
	vec3f VecFacing, VecStrafe, BoltPos;
	BoltPos = Position;

	VecFacing = Velocity;
	VecFacing.Normalize();

	BoltPos  += VecFacing * m_Bolt.z;			//move fwd / back
	VecStrafe = VecFacing.CrossProduct(m_Up);	
	BoltPos  += VecFacing * m_Bolt.x;			//move left / right
	BoltPos  += m_Up * m_Bolt.y;				//move up / down


	//Calculate spring dynamics for smoother, nicer camera
	m_TheSpring.Solve(m_SpringForce);
	m_TheSpring.Update(m_SpringDeltaT, m_SpringFriction);
	m_TheSpring.AnchorLeft(BoltPos);

	//Position camera on other end of spring and point at LookHere
	m_Position = m_TheSpring.GetRight();
	m_Lookat = LookHere;

}

//-------------------------------------------------------------------------- CHASE
//Spring dampened.
//Spring tied to position/object directly, no need for any position etc.
//Chase infers we want object being chased as subject, so we'll look at it too.
//--------------------------------------------------------------------------------
void CCamera::SpringDampenedChase(vec3f &Position)
{

	//Calculate spring dynamics for smoother, nicer camera
	m_TheSpring.Solve(m_SpringForce);
	m_TheSpring.Update(m_SpringDeltaT, m_SpringFriction);
	m_TheSpring.AnchorLeft(Position);

	//Position camera on other end of spring and point at LookHere
	m_Position = m_TheSpring.GetRight();
	m_Lookat = Position;

	m_SpringForce.Set(0,0,0);
}


//------------------------------------------------------------------------ PUSHING
// Only works with spring dampened models.
// these methods will 'push' the camera around the lookat subject
//--------------------------------------------------------------------------------
//------------------------------------------------------------------------ PUSHING
// Left / right
//--------------------------------------------------------------------------------
void CCamera::PushSideways(float amount)
{
	vec3f VecFacing, VecStrafe;

	//get side to side vector so we can strafe along it.
	VecFacing = m_Lookat - m_Position;
	VecFacing.Normalize();
	VecStrafe = VecFacing.CrossProduct(m_Up);

	m_SpringForce = (VecStrafe * amount);

}
//------------------------------------------------------------------------ PUSHING
// push up / down
//--------------------------------------------------------------------------------
void CCamera::PushVert(float amount)
{
	m_SpringForce = (m_Up * amount);

}

//------------------------------------------------------------------------ PUSHING
// push towards / away
//--------------------------------------------------------------------------------
void CCamera::PushIn(float amount)
{
	vec3f VecFacing;

	//get side to side vector so we can strafe along it.
	VecFacing = m_Lookat - m_Position;
	VecFacing.Normalize();

	m_SpringForce = (VecFacing * amount);

}

//------------------------------------------------------------------------ PUSHING
// push in this direction - camera super coords.
//--------------------------------------------------------------------------------
void CCamera::Push(vec3f &amount)
{
	m_SpringForce = amount;

}


//--------------------------------------------------------------------------------
// a selection of gets and sets.
//--------------------------------------------------------------------------------

//get and set bits
vec3f CCamera::GetPosition(){ return m_Position; }
vec3f CCamera::GetLookat()	{ return m_Lookat; }
vec3f CCamera::GetUp()		{ return m_Up; }
