#ifndef	_CAMERA_H_
#define	_CAMERA_H_

#define TRUE 1
#define FALSE 0

#include "..\Maths\maths.h"
#include "..\Objects\Object.h"
#include "spring.h"


enum camRotationMatrix
{
	MATRIX_MODELVIEW = 0,
	MATRIX_PROJECTION,
	MATRIX_ROTATION,	

};

enum camVariable
{
	CAMERA_POSITION = 0,
	CAMERA_LOOKAT,	
	CAMERA_UP,
	CAMERA_VELOCITY,
	CAMERA_BOLT,
	CAMERA_VELOCITY_MAX,
	CAMERA_ZPLANE_FAR,
	CAMERA_ZPLANE_NEAR,
	CAMERA_ZOOM,
	CAMERA_TWIST
};


class CCamera : CObject
{
	vec3f			m_Up,
					m_Slide;

	CSingleSpring	m_TheSpring;
	float			m_CurrentAngle,
					m_SpringDeltaT,
					m_SpringFriction;
	vec3f			m_Bolt,
					m_SpringForce;

	vec3f			m_Lookat,
					m_Velocity;

	mat4f			m_RotationMatrix,
					m_ViewMatrix,
					m_ProjectionMatrix;

	float			m_Vmax;

	float			m_FOV, m_Aspect, m_ZNear, m_ZFar;

public:

	//constructor
	CCamera();

	//get and set bits
	vec3f GetPosition();
	vec3f GetLookat();
	vec3f GetUp();
	float GetPitch();
	float GetYaw();

	bool Set(camVariable _me, float _tothis);
	bool Set(camVariable _me, vec3f &_tothis);
	bool Change(camVariable _me, float _amount);
	bool Change(camVariable _me, vec3f &_amount);

	void BuildMatrix();
	void GetMatrix(camRotationMatrix mtx, mat4f &matrix);
	void GetMatrixNoBuild(camRotationMatrix mtx, mat4f &matrix);
   mat4f GetMatrix(camRotationMatrix mtx);
   mat4f GetMatrixNoBuild(camRotationMatrix mtx);

	void SetCamera(float x, float y, float z,
					float vx, float vy, float vz,
					float ux, float uy, float uz);
	void SetProjection(float _fov, float _aspectRatio, float _zNear, float _zFar);
	void SetSpring(float k_coefficient, float length, float friction, float deltaT);

	// some updatey functions.
	void ModifyVelocity(float plusx, float plusy, float plusz);
	void Update(vec3f direction, float amount);
	void Advance(float amount);
	void Strafe(float amount);
	void Ascend(float amount);
	void Rotate(float AngleDir, float amountx, float amounty, float amountz);
	void RotateYX(float y, float x, float scale);

	//Update Functions -most likely ones to be called from outside
	void UpdateUsingVelocity(float timelapsed, float friction);
	void Track_BoltOn(vec3f &Position, vec3f &Velocity, vec3f &LookHere);
	void SpringDampenedBolt(vec3f &Position, vec3f &Velocity, vec3f &LookHere);
	void SpringDampenedChase(vec3f &Position);

	void PushSideways(float amount);
	void PushVert(float amount);
	void PushIn(float amount);
	void Push(vec3f &amount);

	bool Init()			{ return 1; }
	int  Update( void* ){ return 1; }
	bool Destroy()		{ return 1; }

};

#endif
