#pragma warning(disable:4786)

#include "spring.h"


//================================================================================
// base Mass class, used for springs+
//================================================================================
CMassCentre::CMassCentre()
{
	m_Mass = 0;
	m_Force.Set(0.0f,0.0f,0.0f);
	m_Position.Set(0.0f,0.0f,0.0f);
	m_Velocity.Set(0.0f,0.0f,0.0f);
}


void CMassCentre::ResetForce()
{
	this->m_Force.Set(0.0f,0.0f,0.0f);
}


void CMassCentre::ApplyForce(vec3f outsideForce)
{
	this->m_Force += outsideForce;
}

//standard update function
void CMassCentre::Update(float deltaT, float friction)
{
	this->m_Velocity += ((this->m_Force) / this->m_Mass);
	this->m_Velocity *= friction;
	this->m_Position += (this->m_Velocity * deltaT);
}

//act as if attached to the end of a spring
//		  K = spring coefficient (springiness 0 - 1)
//deltaDist = rest length of spring
void CMassCentre::SpringForce(vec3f &otherEnd, float K, float deltaDist, vec3f &fout)
{
	vec3f	forceDir;
	float	distance,
			delta,
			intensity;

	//which way to spring?
	forceDir = otherEnd-this->m_Position;

	distance = forceDir.Length();

	if (distance < SPRINGTOLERANCE)
	{
		fout.Set(0.0f,0.0f,0.0f);
		this->m_Force += fout;	
		return;
	}

	forceDir.Normalize();

	//work out how much force is acting, ie how much to spring by
	delta = distance-deltaDist;
	intensity = K * delta;

	fout = forceDir*intensity;
	this->m_Force += fout;

//	PRINT("[%f,%f,%f]\t", this->m_Force.x,this->m_Force.y,this->m_Force.z);
}


void CMassCentre::SetForceInvert(vec3f &force)
{
	this->m_Force += (force * (-1.0f));
}



//gets and sets
void CMassCentre::SetPosition(vec3f &tothis)	{ this->m_Position = tothis; }
void CMassCentre::SetMass(float m)				{ this->m_Mass = m; }
void CMassCentre::SetPosition(float x, float y, float z)
{
	this->m_Position.x = x;
	this->m_Position.y = y;
	this->m_Position.z = z;
}

vec3f CMassCentre::GetPosition()				{return this->m_Position; }
float CMassCentre::GetMass()					{return this->m_Mass; }



//================================================================================
// Springs
//================================================================================

CSpring::CSpring()
{
	K = 0.8f;
	length = 1.0f;
}

void CSpring::SetEnds(CMassCentre *left, CMassCentre *right)
{
	m_Ends[0] = left;
	m_Ends[1] = right;
}


void CSpring::Set(float k_coefficient, float sprlength)
{
	this->K				= k_coefficient;
	this->length		= sprlength;
}

void CSpring::SolveForces()
{
	vec3f temp;
//	PRINT("\n");
	m_Ends[0]->SpringForce(m_Ends[1]->GetPosition(), this->K, this->length, temp);
	//m_Ends[1]->SpringForce(m_Ends[0]->m_Position, this->K, this->length, temp);
	m_Ends[1]->SetForceInvert(temp);

}

void CSpring::ApplyForce(vec3f &force)
{
	m_Ends[0]->ApplyForce(force);
	m_Ends[1]->ApplyForce(force);

}



CSingleSpring::CSingleSpring()
{

	this->SetEnds(&left, &right);
	left.SetMass(10.0f);
	right.SetMass(10.0f);
	left.SetPosition(0,0,0);
	right.SetPosition(0,0,0);
}

void CSingleSpring::AnchorLeft(vec3f &toHere)
{
	left.SetPosition(toHere);

}

void CSingleSpring::Solve()
{

	left.ResetForce();
	right.ResetForce();
	
	SolveForces();

}

void CSingleSpring::Solve(vec3f &OutsideForce)
{
	left.ResetForce();
	right.ResetForce();
	
	SolveForces();

	left.ApplyForce(OutsideForce);
	right.ApplyForce(OutsideForce);

}

void CSingleSpring::Update(float deltaT, float friction)
{
	left.Update(deltaT,  friction);
	right.Update(deltaT,  friction);
}

vec3f CSingleSpring::GetLeft()
{
	return left.GetPosition();
}

vec3f CSingleSpring::GetRight()
{
	return right.GetPosition();
}


