#include "Collisions.h"

// From Rich's code
bool SphereSphereSweepInfinite(vec3f &_pos1, vec3f &_pos2, vec3f &_vel1, vec3f &_vel2, float _rad1, float _rad2)
{
	vec3f Vr = _vel1 - _vel2;
	vec3f V1 = _pos2 - _pos1;

	float rSq = (_rad1 + _rad2) * (_rad1 + _rad2);

	float l = Vr.DotProduct(V1)/Vr.Length();

	if (l <= 0) return false;

	float sSq = V1.DotProduct(V1) - (l * l);

	if (sSq >= rSq) return false;
	
	return true;
}

const bool SphereSphereDynamicStatic
(
vec3f &Ap, //current position of sphere A
vec3f &Av, //current velocity of sphere A
float ra, //radius of sphere A
vec3f &Bp, //current position of sphere B
float rb, //radius of sphere B
vec3f &NewCentre //position of centre of A at collision
)
{	
	// A Moves far enough?
	float RadAB = ra + rb;
	vec3f AB	= Bp - Ap;
	float Dist	= AB.Length();

	if( AB.Length() <= RadAB*RadAB )				//check if they're currently overlapping
	{       
		return true;
	}

	if( Av.Length() < (Dist - RadAB) )
	{ 
		return false; 
	}

	// A Is moving towards B?
	vec3f N = Av;
	N.Normalize();

	vec3f C = AB;	
	float  D = N.DotProduct(C);
	if(D <= 0)			// Vector points away
	{
		return false;
	}

	// A Ever gets close enough?
	float dist = C.Length();
	float F = (Dist * Dist) - (D * D);
	float RadABsq = RadAB * RadAB;

	if(F >= RadABsq)	//A sails past B
	{
	  return false;
	}

	//
	float T = RadABsq - F;
	if(T < 0)			//Triangle doesn't exist
	{
		return false;
	}

	//
	float distToColl = D - float(sqrt(T));
	float mag = Av.Length();
	if(mag < distToColl)
	{
		return false;
	}

	NewCentre = Av;
	NewCentre.Normalize();
	NewCentre *= distToColl;

	return true; 
}

const bool SphereSphereSweep(vec3f &Ap, vec3f &Av, float ra, vec3f &Bp, vec3f &Bv, float rb, float &DistAlong)
{
	Av -= Bv;

	// A Moves far enough?
	float RadAB = ra + rb;
	vec3f AB	= Bp - Ap;
	float Dist	= AB.Length();

	if( AB.Length() <= RadAB*RadAB )				//check if they're currently overlapping
	{       
		DistAlong = 0;
		return true;
	}

	if( Av.Length() < (Dist - RadAB) )
	{ 
		return false; 
	}

	// A Is moving towards B?
	vec3f N = Av;
	N.Normalize();

	vec3f C = AB;	
	float  D = N.DotProduct(C);
	if(D <= 0)			// Vector points away
	{
		return false;
	}

	// A Ever gets close enough?
	float dist = C.Length();
	float F = (Dist * Dist) - (D * D);
	float RadABsq = RadAB * RadAB;

	if(F >= RadABsq)	//A sails past B
	{
	  return false;
	}

	//
	float T = RadABsq - F;
	if(T < 0)			//Triangle doesn't exist
	{
		return false;
	}

	//
	float distToColl = D - float(sqrt(T));
	float mag = Av.Length();
	if(mag < distToColl)
	{
		return false;
	}

	vec3f NewCentre = Av;
	NewCentre.Normalize();
	NewCentre *= distToColl;

	DistAlong = NewCentre.Length() / Av.Length();

	return true; 
}


void BounceSpheres(	vec3f &Ap,	// Current position of sphere A
					vec3f &Av,	// Current velocity of sphere A
					float Amass,// Mass of A
					vec3f &Anv,	// New velocity of A
					vec3f &Bp,	// Current position of sphere B
					vec3f &Bv,	// Current velocity of sphere B
					float Bmass,// Mass of B
					vec3f &Bnv	// New velocity of B
					)
{
	vec3f N = Ap - Bp;
	N.Normalize();

	float a1 = Av.DotProduct(N);
	float a2 = Bv.DotProduct(N);

	float optimizedP = (2.0f * (a1 - a2)) / (Amass + Bmass);

	Anv = Av - optimizedP * Bmass * N;
	Bnv = Bv + optimizedP * Amass * N;
}
