/********************
*                   *
*   NeHeGL Header   *
*                   *
**********************************************************************************
*                                                                                *
*	You Need To Provide The Following Functions:                                 *
*                                                                                *
*	BOOL Initialize (GL_Window* window, Keys* keys);                             *
*		Performs All Your Initialization                                         *
*		Returns TRUE If Initialization Was Successful, FALSE If Not              *
*		'window' Is A Parameter Used In Calls To NeHeGL                          *
*		'keys' Is A Structure Containing The Up/Down Status Of keys              *
*                                                                                *
*	void Deinitialize (void);                                                    *
*		Performs All Your DeInitialization                                       *
*                                                                                *
*	void Update (DWORD milliseconds);                                            *
*		Perform Motion Updates                                                   *
*		'milliseconds' Is The Number Of Milliseconds Passed Since The Last Call  *
*		With Whatever Accuracy GetTickCount() Provides                           *
*                                                                                *
*	void Draw (void);                                                            *
*		Perform All Your Scene Drawing                                           *
*                                                                                *
*********************************************************************************/

#ifndef GL_FRAMEWORK__INCLUDED
#define GL_FRAMEWORK__INCLUDED

#include <winsock2.h>		//! Needed, to stop multiple definitions (used in netwok code)  
							//  This NEEDS to be included before windows.h, shouldn't really 
							//  be here, but it's doing no harm...

#include <windows.h>											// Header File For Windows
#include <gl\gl.h>												// Header File For The OpenGL32 Library
#include <gl\glu.h>												// Header File For The GLu32 Library
#include <gl\glaux.h>											// Header File For The GLaux Library

#pragma comment( lib, "opengl32.lib" )							// Search For OpenGL32.lib While Linking
#pragma comment( lib, "glu32.lib" )								// Search For GLu32.lib While Linking
#pragma comment( lib, "glaux.lib" )	

//#ifndef CDS_FULLSCREEN											// CDS_FULLSCREEN Is Not Defined By Some
//#define CDS_FULLSCREEN 4										// Compilers. By Defining It This Way,
//#endif															// We Can Avoid Errors

typedef struct {									// Structure For Keyboard Stuff
	BOOL keyDown [256];								// Holds TRUE / FALSE For Each Key
	BOOL toggled [256];								// Holds TRUE / FALSE For Each Key
} Keys;												// Keys

class Mouse {
public:
	BOOL Right;	
	BOOL Left;	
	BOOL Middle;	
	
	int XPos;
	int YPos;
	int XMove;
	int YMove;	
	int ZMove;	

	Mouse()
	{
		Right	=	Left	=	Middle	= false;		
		XPos	=	YPos	= 
		XMove	=	YMove	=	ZMove	= 0;	
	}

	void Update(int _x, int _y)
	{
		XMove	= _x - XPos;
		YMove	= _y - YPos;
		XPos	= _x;
		YPos	= _y;
	}

	void BlankMove()
	{
		XMove	=	YMove	=	ZMove	= 0;
	}
};	


typedef struct {									// Contains Information Vital To Applications
	HINSTANCE		hInstance;						// Application Instance
	const char*		className;						// Application ClassName
} Application;										// Application

typedef struct {									// Window Creation Info
	Application*		application;				// Application Structure
	char*				title;						// Window Title
	int					width;						// Width
	int					height;						// Height
	int					bitsPerPixel;				// Bits Per Pixel
	BOOL				isFullScreen;				// FullScreen?
} GL_WindowInit;									// GL_WindowInit

class GL_Window
{													// Contains Information Vital To A Window
public:
	Keys*				keys;						// Key Structure
	Mouse*				mouse;						// Mouse Structure
	HWND				hWnd;						// Window Handle
	HDC					hDC;						// Device Context
	HGLRC				hRC;						// Rendering Context
	GL_WindowInit		init;						// Window Init
	int					winwidth;
	int					winheight;
	BOOL				isVisible;					// Window Visible?
	DWORD				lastTickCount;				// Tick Counter

	GLuint	base;				// Base Display List For The Font Set
	GLfloat	cnt1;				// 1st Counter Used To Move Text & For Coloring
	GLfloat	cnt2;				// 2nd Counter Used To Move Text & For Coloring

	GLvoid BuildFont(GLvoid);
	GLvoid KillFont(GLvoid);
	GLvoid glPrint(const char *fmt, ...);

};													// GL_Window

void TerminateApplication (GL_Window* window);		// Terminate The Application

void ToggleFullscreen (GL_Window* window);			// Toggle Fullscreen / Windowed Mode

// These Are The Function You Must Provide
BOOL Initialize (GL_Window* window, Keys* keys, Mouse* mouse);	// Performs All Your Initialization

void Deinitialize (void);							// Performs All Your DeInitialization

void LocalInput();
int  Update (DWORD milliseconds);					// Perform Motion Updates
void Draw (void);									// Perform All Your Scene Drawing

void StartFrame();
void EndFrame();
void OrthoMode(int left, int top, int right, int bottom);
void PerspectiveMode();

inline void Toggle(bool &_v)
{
	_v = (_v == true) ? false : true;
}

#endif												// GL_FRAMEWORK__INCLUDED


