#include "../../Globals.h"
#include "CEmitter.h"

#define NUM_PARTS_DEBUG	(1000)

CEmitter::CEmitter(CStaticList<CParticle> *_pile)
{
	m_List = _pile;
}

CEmitter::~CEmitter()
{

}

bool CEmitter::Init()
{

	m_Position.Set(0,0,0);
	m_Rotation.Set(0,0,0);
	m_Scale.Set(0,0,0.0);
	m_Velocity.Set(0,0,0);

	m_NumParticles	= NUM_PARTS_DEBUG;
	m_Loops			= 0;
	m_Visible		= true;
	m_Alive			= true;
	m_DontDestroy	= false;

	m_Speed			= 0.55f;		m_SpeedVar		= 0.005f;
	m_Mass			= 1.0f;			m_MassVar		= 0.75f;
	m_Elasticity	= -0.66f;		m_ElasticityVar	= 0.25f;
	m_ScaleP		= 0.250f;		m_ScalePVar		= .1f;
	m_Age			= 1;			m_AgeVar		= 1;
	m_Collides		= true;
	m_Colour.Set(0.0f, 0.0f, 0.0f, 0.5f);
	m_ColourVar.Set(0.2f, 0.2f, 0.2f, 0.2f);
	m_EndColour.Set(1.0f, 1.0f, 1.0f, 0.0f);
	m_Texture		= g_Renderer.m_PTex;

	m_LocalForce.Set(0.0f, -0.010f, 0.0f);
	m_Friction = 1.0f;
	
	m_Up.Set(0.0f, -1.0f, 0.0f);
	m_Right.Set(1.0f, 0.0f, 0.0f);
	m_Front.Set(0.0f, 0.0f, -1.0f);

	// get some particles to play with
	int i	= 0;
	
	while(i++ < m_NumParticles)
		m_List->Add();
	m_pStart = m_List->GetHead();	

	
	CStaticList<CParticle>::CSLnode * temp;
	temp = m_pStart;
	i = 0;
	while((i++ < m_NumParticles) && (temp != NULL))
	{		
		temp->data.m_Deaths = 0;
		Emit(temp);
		temp = temp->next;
	}

	return true;
}

bool CEmitter::Restart()
{
	CStaticList<CParticle>::CSLnode * temp;
	temp = m_pStart;
	int i = 0;
	while((i++ < m_NumParticles) && (temp != NULL))
	{		
		int d = temp->data.m_Deaths;
		temp->data.m_Deaths = 0;

		if(d > m_Loops)
		{
			Emit(temp);
			temp->data.m_Age = random() * m_Age;		
		}
		temp = temp->next;
	}

	return true;
}


int  CEmitter::Update(void* _arg)
{	
	if(!m_Alive) return false;

	float tp = float(*(double*) _arg);
	m_Position+=m_Velocity*tp;

	CStaticList<CParticle>::CSLnode * temp;
	temp = m_pStart;

	int i = 0, dead = 0;
	while((i++ < m_NumParticles) && (temp != NULL))
	{
		temp->data.ApplyForce(m_LocalForce);
		int ret = temp->data.Update(_arg);
		if( ret == born_again)
		{
			if(!Emit(temp))
				dead++;
		}
		temp = temp->next;
	}
	
	// are all the particles really dead?
	if(dead == (m_NumParticles))
		return 0;

	return 1;
}

bool CEmitter::Emit(CStaticList<CParticle>::CSLnode *_me)
{
	if( (m_Loops != -1) && (_me->data.m_Deaths > m_Loops) )
		return false;
	
	// Position
	_me->data.m_Position.x = m_Position.x + ( (random()-0.5f) * m_Scale.x );
	_me->data.m_Position.y = m_Position.y + ( (random()-0.5f) * m_Scale.y );
	_me->data.m_Position.z = m_Position.z + ( (random()-0.5f) * m_Scale.z );
	
	//set velocity
	vec3f newrand;
	newrand.Set(random()-0.5f,random()-0.5f,random()-0.5f);
	newrand.Normalize();
	_me->data.m_Velocity.x	= ( newrand.x ) * m_Speed + ( (random()-0.5f) * m_SpeedVar );	
	_me->data.m_Velocity.y	= ( newrand.y ) * m_Speed + ( (random()-0.5f) * m_SpeedVar );	
	_me->data.m_Velocity.z	= ( newrand.z ) * m_Speed + ( (random()-0.5f) * m_SpeedVar );	
	
	//age
	_me->data.m_Age			= m_Age			+ ( (random()-0.5f) * m_AgeVar );
	_me->data.m_Alive		= m_Alive;
	_me->data.m_OneOverAge	= _me->data.m_Age;
	
	//pysical attributes
	_me->data.m_Mass		= m_Mass		+ ( (random()-0.5f) * m_MassVar );
	_me->data.m_Elasticity	= m_Elasticity	+ ( (random()-0.5f) * m_ElasticityVar );
	_me->data.m_Scale.x		= m_ScaleP		+ ( (random()-0.5f) * m_ScalePVar );
	
	_me->data.m_Collide		= m_Collides;
	
	//appearance
	_me->data.m_StartColour.r	= m_Colour.r	+ ( (random()-0.5f) * m_ColourVar.r );
	_me->data.m_StartColour.g	= m_Colour.g	+ ( (random()-0.5f) * m_ColourVar.g );
	_me->data.m_StartColour.b	= m_Colour.b	+ ( (random()-0.5f) * m_ColourVar.b );
	_me->data.m_StartColour.a	= m_Colour.a	+ ( (random()-0.5f) * m_ColourVar.a );
	
	_me->data.m_EndColour.r		= m_EndColour.r;
	_me->data.m_EndColour.g		= m_EndColour.g;
	_me->data.m_EndColour.b		= m_EndColour.b;
	_me->data.m_EndColour.a		= m_EndColour.a;
	_me->data.m_Texture			= m_Texture;
	
	return true;
}

void CEmitter::Render(bool _reflected)
{
	CStaticList<CParticle>::CSLnode * temp;
	temp = m_pStart;

//	if(_reflected)
//		m_Front.y *= -1;

	int i = 0;
	while((i++ < m_NumParticles) && (temp != NULL))
	{
		temp->data.Render(g_Renderer.m_Up, g_Renderer.m_Right, g_Renderer.m_Face);
		temp = temp->next;
	}
}


void CEmitter::RenderLines()
{
	CStaticList<CParticle>::CSLnode * temp;
	temp = m_pStart;

	int i = 0;
	while((i++ < m_NumParticles) && (temp != NULL))
	{
		vec3f p = temp->data.m_Position;
		vec3f v = -1.0f * temp->data.m_Velocity;
		col4f c = temp->data.m_Colour;

//		glLineWidth(10);//temp->data.m_Scale.y);
		
		glColor4f(c.r, c.g, c.b, c.a);
		glBegin(GL_LINES);
		{
			glVertex3f(p.x, p.y, p.z);
			glVertex3f(p.x+v.x, p.y+v.y, p.z+v.z);
		}
		glEnd();

		temp = temp->next;
	}

}

bool CEmitter::Destroy()
{
	CStaticList<CParticle>::CSLnode * temp, *t2;
	temp = m_pStart;

	int i=0;
	while((i++ < m_NumParticles) && (temp != NULL))
	{
		t2 = temp->next;
		m_List->Remove(temp);
		temp = t2;
	}

	return true;
}
