#include "../../Globals.h"
#include "CTrail.h"

CTrailPoint::CTrailPoint()
{
	m_Position.Set(0,0,0);
	m_SlideVec.Set(0,1,0);
}

void CTrailPoint::SetPos(vec3f &_v)
{
	m_Position = _v;
}

//-------------------------------------------------
CTrail::CTrail()
{

}
CTrail::~CTrail()
{
	Destroy();
}

bool CTrail::Init(float _lengthInUnits, int _resolution, unsigned int _texture, vec3f &_pos, col4f &_basecol, vec3f &_scale)
{
	m_Scale.Set		  (_scale);
	m_Resolution	= _resolution;
	m_ResMul		= 1.0f / _resolution;
	m_DistApart_Sq	= (_lengthInUnits / float(m_Resolution)) * (_lengthInUnits / float(m_Resolution));
	m_Point			= new CTrailPoint[m_Resolution];
	m_Colour		= new		 col4f[m_Resolution];
	m_Texture		= _texture;
	m_LastPoint		= 0;
	m_Position.Set	  (_pos);
	

	for (float i =0; i<_resolution; i++)
	{
//		m_Colour[int(i)].Set(0.0f,0.0f,0.0f,(m_Resolution-i)/m_Resolution);
		m_Colour[int(i)].Set(_basecol.r, _basecol.g, _basecol.b, _basecol.a * (m_Resolution-i)/m_Resolution);
		m_Point[int(i)].SetPos (_pos);
	}

	return true;
}



bool CTrail::Init()
{

	return 1;
}
bool CTrail::UpdateFlat(vec3f &_vel)
{
	vec3f v = _vel;
	vec3f up(0,1,0);
	v.Normalize();

	vec3f AB = m_Position - m_Point[m_LastPoint].m_Position;
	float d = AB.LengthNoRoot();

	if(d > m_DistApart_Sq) //Add new
	{
		if((m_LastPoint++) >= m_Resolution-1) 
			m_LastPoint = 0;

		m_Point[m_LastPoint].SetPos(m_Position);
		m_Point[m_LastPoint].m_SlideVec.Set(v.CrossProduct(up));
	//	Build(_camPosition);
		return true;

	}
	//else don't add new
	

	return false;
}

int  CTrail::Update( void* )
{
/*
	vec3f AB = m_Position - m_Point[m_LastPoint].m_Position;
	float d = AB.LengthNoRoot();

	if(d > m_DistApart_Sq) //Add new
	{
		if((m_LastPoint++) >= m_Resolution-1) 
			m_LastPoint = 0;

		m_Point[m_LastPoint].SetPos(m_Position);
	//	Build(_camPosition);
		return true;

	}
	//else don't add new
*/	

	return 1;
}

void CTrail::RenderLines()
{
	int n, a=0;

	glEnable(GL_BLEND);
	glBegin(GL_LINE_STRIP);
	for(n = m_LastPoint; n > 0; n--)
	{		
		glColor4f(m_Colour[a].r, m_Colour[a].g, m_Colour[a].b, m_Colour[a++].a);
		glVertex3f(m_Point[n].m_Position.x, m_Point[n].m_Position.y, m_Point[n].m_Position.z);
	}
	for(n = m_Resolution-1; n > m_LastPoint; n--)
	{
		glColor4f(m_Colour[a].r, m_Colour[a].g, m_Colour[a].b, m_Colour[a++].a);
		glVertex3f(m_Point[n].m_Position.x, m_Point[n].m_Position.y, m_Point[n].m_Position.z);
	}
	glEnd();
	glDisable(GL_BLEND);

}

void CTrail::Render()
{
	int n, a=0;

	glNormal3f( 0.0f, 1.0f, 0.0f);

	//glDisable( GL_CULL_FACE );
	glDisable( GL_DEPTH_TEST );
	glEnable(GL_BLEND);
	glBegin(GL_TRIANGLE_STRIP);
	for(n = m_LastPoint; n > 0; n--)
	{		
		glColor4f(m_Colour[a].r, m_Colour[a].g, m_Colour[a].b, m_Colour[a++].a);
		float scale = 1.0f - float(a) * m_ResMul;

		glTexCoord2f( 0.0f, 0.0f );
		glVertex3f( m_Point[n].m_Position.x + m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y + m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z + m_Point[n].m_SlideVec.z * scale * m_Scale.z);

		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( m_Point[n].m_Position.x - m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y - m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z - m_Point[n].m_SlideVec.z * scale * m_Scale.z);

	}
	for(n = m_Resolution-1; n > m_LastPoint; n--)
	{
		glColor4f(m_Colour[a].r, m_Colour[a].g, m_Colour[a].b, m_Colour[a++].a);

		float scale = 1.0f - float(a) * m_ResMul;

		glTexCoord2f( 0.0f, 0.0f );
		glVertex3f( m_Point[n].m_Position.x + m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y + m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z + m_Point[n].m_SlideVec.z * scale * m_Scale.z);

		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( m_Point[n].m_Position.x - m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y - m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z - m_Point[n].m_SlideVec.z * scale * m_Scale.z);	
	}
	glEnd();
	glDisable(GL_BLEND);
	glEnable( GL_DEPTH_TEST );
	//glEnable( GL_CULL_FACE );
}


bool CTrail::Destroy()
{
	if(m_Point)	 delete[] m_Point;
	if(m_Colour) delete[] m_Colour;
	m_Point=NULL;
	m_Colour=NULL;

	return 1;
}

//-------------------------------------------------------------

bool CTrailTimed::Init()
{

	return 1;
}

bool CTrailTimed::Init(float _lengthInSeconds, int _resolution, unsigned int _texture, vec3f &_pos, col4f &_basecol, vec3f &_scale)
{

	m_TimeApart		= _lengthInSeconds / float(_resolution); //time in ms;
	m_Timer			= 0;
	
	m_Scale.Set		 (_scale);
	m_Resolution	= _resolution;
	m_ResMul		= 1.0f / _resolution;
	m_Point			= new CTrailPoint[m_Resolution];
	m_Colour		= new		 col4f[m_Resolution];
	m_Texture		= _texture;
	m_LastPoint		= 0;
	m_Position.Set	 (_pos);
	

	for (float i =0; i<_resolution; i++)
	{
		m_Colour[int(i)].Set(_basecol.r, _basecol.g, _basecol.b, _basecol.a * (m_Resolution-i)/m_Resolution);
		m_Point[int(i)].SetPos (_pos);
	}


	return true;
}

int  CTrailTimed::Update(void *_arg)
{
	if(!_arg) return false;	//make sure we have a time passed

	double tp = *(double*) _arg;

	m_Timer += tp;
	
	//add new point?
	if(m_Timer > m_TimeApart)
	{
		m_Timer = 0;
		vec3f pA = m_Point[m_LastPoint].m_Position;

		if((m_LastPoint++) >= m_Resolution-1) 
			m_LastPoint = 0;

		m_Point[m_LastPoint].SetPos(m_Position);

		vec3f v = m_Point[m_LastPoint].m_Position - pA;
		vec3f up(0,1,0);
		v.Normalize();

		m_Point[m_LastPoint].m_SlideVec.Set(v.CrossProduct(up));
	//	Build(_camPosition);
		return 1;

	}
	//else don't add new

	return 0;
}

bool CTrailTimed::Destroy()
{
	if(m_Point)	 delete[] m_Point;
	if(m_Colour) delete[] m_Colour;
	m_Point=NULL;
	m_Colour=NULL;

	return 1;
}

void CTrailTimed::Render()
{
	int n, a=0;


	glNormal3f( 0.0f, 1.0f, 0.0f);

	glDisable( GL_CULL_FACE );
	glDisable( GL_DEPTH_TEST );
	glEnable(GL_BLEND);
	glBegin(GL_TRIANGLE_STRIP);
	for(n = m_LastPoint; n > 0; n--)
	{		
		glColor4f(m_Colour[a].r, m_Colour[a].g, m_Colour[a].b, m_Colour[a++].a);
	
		// interesting trail patterns
	//	float pv = (1.0f - float(a) * m_ResMul);
	//	float scale = cos(2*sin(20*pv))*pv;
		float scale = (1.0f - float(a) * m_ResMul);

		glTexCoord2f( 0.0f, 0.0f );
		glVertex3f( m_Point[n].m_Position.x + m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y + m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z + m_Point[n].m_SlideVec.z * scale * m_Scale.z);

		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( m_Point[n].m_Position.x - m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y - m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z - m_Point[n].m_SlideVec.z * scale * m_Scale.z);

	}
	for(n = m_Resolution-1; n > m_LastPoint; n--)
	{
		glColor4f(m_Colour[a].r, m_Colour[a].g, m_Colour[a].b, m_Colour[a++].a);

		float scale = (1.0f - float(a) * m_ResMul);

		glTexCoord2f( 0.0f, 0.0f );
		glVertex3f( m_Point[n].m_Position.x + m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y + m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z + m_Point[n].m_SlideVec.z * scale * m_Scale.z);

		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( m_Point[n].m_Position.x - m_Point[n].m_SlideVec.x * scale * m_Scale.x, 
					m_Point[n].m_Position.y - m_Point[n].m_SlideVec.y * scale * m_Scale.y, 
					m_Point[n].m_Position.z - m_Point[n].m_SlideVec.z * scale * m_Scale.z);	
	}
	glEnd();
	glDisable(GL_BLEND);
	glEnable( GL_DEPTH_TEST );
	glEnable( GL_CULL_FACE );
}
