#include "../Globals.h"
#include "Controller.h"

double c = 0.0;

CWorldController::CWorldController()
{
	m_FPS = 0;
	g_Timer.Init();
}

bool CWorldController::Init(CPlayer *_1up, CPlayer *_2up, CLevel *_level)
{
	m_Player1	= _1up;
	m_Player2	= _2up;
	m_Level		= _level;

	float f = 0.1f;
	m_Player1->	Init(vec3f(-6.5f, 0.0f, 0.0f), col4f(1.0f, 0.0f, 0.0f, 1.0f), X, Up);
	m_Player2->	Init(vec3f( 6.5f, 0.0f, 0.0f), col4f(0.0f, 0.0f, 1.0f, 1.0f), X, Down);
	m_Puck.		Init(vec3f( random()*f-f, 0.0f, random()*f-f), col4f(0.1f, 0.1f, 0.1f, 1.0f));

	m_1UP_AI	= false;
	m_2UP_AI	= true;	

	g_FXManager.Init();
	int Em = g_FXManager.AddParticleSys("", vec3f(0,0,0));
	m_GP_Emitter = g_FXManager.GetEmitter(Em);
	m_GP_Emitter->m_DontDestroy = true;

	g_NetSys.Init();

	return true;
}

int CWorldController::Update()
{
	double tp = g_Timer.Tick();
	m_FPS = 1.0f / float(tp);

	if(m_1UP_AI) m_Player1->AIThink(m_Puck.m_Position, m_Puck.m_Velocity);
	if(m_2UP_AI) m_Player2->AIThink(m_Puck.m_Position, m_Puck.m_Velocity);

	void *xm = &tp;
	m_Level->Update(xm);
	m_Player1->Update(xm);
	m_Player2->Update(xm);
	m_Puck.Update(xm);

	g_FXManager.Update(xm);

	// Network
	// Send 1 packet/second
	if((c+=tp) > 1)
	{
		c=0;
		static double timeNow = 0.0;
		timeNow += 0.01;
		void *t = &timeNow;
		g_NetSys.BuildPacket(t);
		g_NetSys.SendPacket();
	}
	// Test for goals ----------------------------
	// 1up scores
	if((m_Puck.m_Position.x >  m_Level->m_Size.x * 0.5f - m_Puck.m_Scale.x * 0.5f) && (m_Puck.m_Position.z < m_Level->m_2UpGoalWidth *  0.5f - m_Puck.m_Scale.x * 0.5f) && (m_Puck.m_Position.z > m_Level->m_2UpGoalWidth * -0.5f + m_Puck.m_Scale.x * 0.5f) )
	{
		m_GP_Emitter->m_Colour.Set(0.9f, 0.0f, 0.0f, 0.5f);
		m_GP_Emitter->m_Position.Set(m_Puck.m_Position);
		m_GP_Emitter->Restart();

		m_Puck.m_Position.x = m_Player2->m_Position.x - 1* m_Player2->m_Scale.x;
		m_Puck.m_Position.z = m_Player2->m_Position.z;
		m_Puck.Init(m_Puck.m_Position, m_Puck.m_BaseColour);
		m_Player1->m_Score++;
	}

	// 2up scores
	if((m_Puck.m_Position.x <  m_Level->m_Size.x * -0.5f + m_Puck.m_Scale.x * 0.5f) && (m_Puck.m_Position.z < m_Level->m_1UpGoalWidth *  0.5f - m_Puck.m_Scale.x * 0.5f) && (m_Puck.m_Position.z > m_Level->m_1UpGoalWidth * -0.5f + m_Puck.m_Scale.x * 0.5f) )
	{	
		m_GP_Emitter->m_Colour.Set(0.0f, 0.0f, 0.9f, 0.5f);
		m_GP_Emitter->m_Position.Set(m_Puck.m_Position);
		m_GP_Emitter->Restart();

		m_Puck.m_Position.x = m_Player1->m_Position.x + 1*m_Player1->m_Scale.x;
		m_Puck.m_Position.z = m_Player1->m_Position.z;
		m_Puck.Init(m_Puck.m_Position, m_Puck.m_BaseColour);
		m_Player2->m_Score++;	

	}

	// Collide players ----------------------------
	float d;
	if (SphereSphereSweep(	m_Player1->GetPos(), m_Player1->GetVel()*float(tp), m_Player1->GetScale(X)*0.5f, m_Player2->GetPos(), m_Player2->GetVel()*float(tp), m_Player2->GetScale(X)*0.5f, d) )
	{
		vec3f An, Bn;
		BounceSpheres(	m_Player1->GetPos(), m_Player1->GetVel(), 1, An, m_Player2->GetPos(), m_Player2->GetVel(), 1, Bn);
		m_Player1->m_Velocity.Set(An);
		m_Player2->m_Velocity.Set(Bn);		
	}

	// Collide players/Pucks
	if (SphereSphereSweep(	m_Player1->GetPos(), m_Player1->GetVel()*float(tp), m_Player1->GetScale(X)*0.5f, m_Puck.GetPos(), m_Puck.GetVel()*float(tp), m_Puck.GetScale(X)*0.5f, d) )
	{
		vec3f An, Bn;
		BounceSpheres(	m_Player1->GetPos(), m_Player1->GetVel(), 1, An, m_Puck.GetPos(), m_Puck.GetVel(), 0.2f, Bn);
		m_Player1->m_Velocity.Set(An);
		m_Puck.m_Velocity.Set(Bn);		
	}
	// Collide players/Pucks
	if (SphereSphereSweep(	m_Player2->GetPos(), m_Player2->GetVel()*float(tp), m_Player2->GetScale(X)*0.5f, m_Puck.GetPos(), m_Puck.GetVel()*float(tp), m_Puck.GetScale(X)*0.5f, d) )
	{
		vec3f An, Bn;
		BounceSpheres(	m_Player2->GetPos(), m_Player2->GetVel(), 1, An, m_Puck.GetPos(), m_Puck.GetVel(), 0.2f, Bn);
		m_Player2->m_Velocity.Set(An);
		m_Puck.m_Velocity.Set(Bn);		
	}

	// Particles ---------------

	return true;
}

void CWorldController::Render()
{   
	double clipplane[] = {0.0f,-1.0f, 0.0f, 0.0f};

	if(g_Renderer.m_ReflectEnabled)
	{
		{	//Set up for reflections
		 
			glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);	// Don't update color or depth.

			glDisable(GL_DEPTH_TEST);		
			glEnable(GL_STENCIL_TEST);								// Draw 1s into the stencil buffer. 
			glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
			glStencilFunc(GL_ALWAYS, 1, 0xffffffff);
			
			m_Level->RenderStencil();

			glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);		// Re-enable update of color and depth. 
			glStencilFunc(GL_EQUAL, 1, 0xffffffff);  // draw if ==1 
			glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
					
			glEnable(GL_DEPTH_TEST);	
			glEnable(GL_CLIP_PLANE0);			
			glClipPlane(GL_CLIP_PLANE0, clipplane);
		}
		//Render reflections 
		glPushMatrix();
		glScalef(1.0, -1.0, 1.0);
		glEnable(GL_NORMALIZE);
		glCullFace(GL_FRONT);

		m_Level->Render(false);
		m_Player1->Render(false);
		m_Player2->Render(false);
		m_Puck.Render(false);

		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		g_FXManager.RenderAll();
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_DEPTH_TEST);
		glEnable(GL_LIGHTING);
    
		{	
			//end reflections
			glDisable(GL_NORMALIZE);
			glCullFace(GL_BACK);	
			glPopMatrix();
			glDisable(GL_CLIP_PLANE0);
		}

		//Render reflective surface
		glEnable(GL_BLEND);
		m_Level->RenderStencil();
		glDisable(GL_BLEND);

		{
			glDisable(GL_STENCIL_TEST);
		}
		
	}//reflections

	//Render real objects
	m_Level->Render(true);
	m_Puck.Render();
	m_Player1->Render();
	m_Player2->Render();

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	g_FXManager.RenderAll();
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);



}


