
#include "../Globals.h"
#include "level.h"

CLevel::CLevel()
{
	m_Size.Set(15.0f, 0.25f, 10.0f);
	m_1UpGoalWidth = m_Size.z * 0.5f;
	m_2UpGoalWidth = m_Size.z * 0.5f;
}

int  CLevel::Update(void *_arg)
{
	if(m_1UpGoalWidth > m_Size.z)	m_1UpGoalWidth = m_Size.z;
	if(m_1UpGoalWidth < 0)			m_1UpGoalWidth = 0;
	if(m_2UpGoalWidth > m_Size.z)	m_2UpGoalWidth = m_Size.z;
	if(m_2UpGoalWidth < 0)			m_2UpGoalWidth = 0;

	return 1;
}

void CLevel::RenderStencil(bool _asSurface)
{
	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
	g_Renderer.RenderPlane(m_Size.x, m_Size.z, _asSurface?1.0f:16.0f, 0);
}

void  CLevel::Render(bool _drawSurface)
{	
	vec3f side = m_Size;
	side.z = 0.25;
	side.y = 0.5;


	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	if(_drawSurface)
	{
		glPushMatrix();
		glTranslatef(0, -0.001f, 0);
		g_Renderer.RenderPlane(m_Size.x, m_Size.z, 16, 0);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-m_Size.x * 0.5f, -0.5f * m_Size.y, 0);
		glRotatef(90, 0, 0, 1);
		g_Renderer.RenderPlane(m_Size.y, m_Size.z, 1, 0);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(m_Size.x * 0.5f, -0.5f * m_Size.y, 0);
		glRotatef(-90, 0, 0, 1);
		g_Renderer.RenderPlane(m_Size.y, m_Size.z, 1, 0);
		glPopMatrix();

		vec3f l(0, 0.01f , -m_Size.z * 0.5f);
		vec3f r(0, 0.01f,  m_Size.z * 0.5f);
		glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
		g_Renderer.RenderLine(l, r, 1);
	}

	// Goals
	vec3f Goal, GoalSide;
	float Offset;

	// 1up
	Goal.Set(0.125, 1.0, m_1UpGoalWidth);	
	GoalSide.Set(0.25f, 0.5f, (m_Size.z - m_1UpGoalWidth) * 0.5f + 0.25f);	
	Offset = GoalSide.z*0.5f + m_1UpGoalWidth *0.5f;

	glPushMatrix();
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glTranslatef(-m_Size.x * 0.5f - GoalSide.x * 0.5f, 0, Offset);
	g_Renderer.RenderCuboid(GoalSide, 0);
	glTranslatef(0, 0, -2.0f * Offset);
	g_Renderer.RenderCuboid(GoalSide, 0);
	glPopMatrix();	

	// 2up
	Goal.Set(0.125, 1.0, m_2UpGoalWidth);	
	GoalSide.Set(0.25f, 0.5f, (m_Size.z - m_2UpGoalWidth) * 0.5f + 0.25f);	
	Offset = GoalSide.z*0.5f + m_2UpGoalWidth *0.5f;

	glPushMatrix();
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glTranslatef(m_Size.x * 0.5f + GoalSide.x * 0.5f, 0, Offset);
	g_Renderer.RenderCuboid(GoalSide, 0);
	glTranslatef(0, 0, -2.0f * Offset);
	g_Renderer.RenderCuboid(GoalSide, 0);
	glPopMatrix();	

	// Left/Right sides
	glPushMatrix();
	glTranslatef(0, 0, m_Size.z * 0.5f + side.z * 0.5f);
	g_Renderer.RenderCuboid(side, 0);
	glTranslatef(0, 0, -2.0f * m_Size.z * 0.5f - side.z);
	g_Renderer.RenderCuboid(side, 0);
	glPopMatrix();	

	// Pillars
	vec3f pillar(0.25f, 2.0f, 0.25f);
	glPushMatrix();
	glTranslatef(m_Size.x * 0.5f - pillar.x * 0.5f + 0.25f, pillar.y * 0.5f + side.y * 0.5f, m_Size.z * 0.5f + side.z * 0.5f);	g_Renderer.RenderCuboid(pillar, 0);
	glTranslatef(m_Size.x * -1.0f + pillar.x - 0.5f, 0, 0);		g_Renderer.RenderCuboid(pillar, 0);
	glTranslatef(0, 0, m_Size.z * -1.0f - side.z);			g_Renderer.RenderCuboid(pillar, 0);
	glTranslatef(m_Size.x * 1.0f - pillar.x + 0.5f, 0, 0);			g_Renderer.RenderCuboid(pillar, 0);
	glPopMatrix();	

}