#include "../Globals.h"
#include "player.h"

bool CPlayer::Init()			
{ 
	m_Position.Set(0,0,0);
	m_Rotation.Set(0,0,0);
	m_Scale.Set(1,1,1);
	m_BaseColour.Set(1,1,1,1);
	m_Score = 0;
	m_BaseLine = 0;
	m_BaseAxis = NONE;
	m_AIDir = Down;
	m_SurfaceFriction = 0.925f;

	m_Trail.Init(1, 10, 0, m_Position, m_BaseColour, m_Scale*0.25);

	return 1; 
}

bool CPlayer::Init(vec3f &_pos, col4f &_col, EDimension _baseDim, EDirection _dir)			
{ 
	m_Position.Set(_pos);
	m_Rotation.Set(0,0,0);
	m_Scale.Set(1,0.125,1);
	m_BaseColour.Set(&_col);
	m_Velocity.Set(0,0,0);	
	m_MaxVelocity.Set(10,10,20);	
	m_MaxAcceleration.Set(60.5f, 20.5f, 80.5f);
	m_Acceleration.Set(0,0,0);
	m_Score = 0;
	m_AIDir = _dir;
	m_SurfaceFriction = 0.925f;

	switch(_baseDim)
	{
	case X:		m_BaseLine = m_Position.x;	break;
	case Z:		m_BaseLine = m_Position.z;	break;
	}
	m_BaseAxis = _baseDim;

	col4f b = m_BaseColour;
	b.a = 0.75;
	m_Trail.Init(0.5f, 40, 0, m_Position, b, m_Scale*0.5);

	return 1; 
}


void CPlayer::AIThink(vec3f &_PuckPos, vec3f &_PuckVel)
{
	int		x = 0, z = 0;	
	float	mval = 0.05f;  //slow puck?

	//Left/Right
	if(_PuckPos.z > m_Position.z + m_Scale.x * 0.5f) { z =  1; }
	if(_PuckPos.z < m_Position.z - m_Scale.x * 0.5f) { z = -1; }

	//up/down
	switch(m_AIDir)
	{
	case Down:
		if((_PuckPos.x > m_Position.x - m_Scale.x) && (_PuckPos.x < m_Position.x ))	
		{ 
			m_Velocity.x = -m_MaxVelocity.x;
		}	
		if( (_PuckVel.x < mval) && (_PuckVel.x > -mval) && (_PuckPos.x >= 0) && (_PuckPos.x < m_Position.x))
		{ 
			m_Velocity.x = -m_MaxVelocity.x; 
		}
		break;

	case Up:
		if((_PuckPos.x < m_Position.x + m_Scale.x) && (_PuckPos.x > m_Position.x ))
		{ 
			m_Velocity.x = m_MaxVelocity.x; 
		}
		if( (_PuckVel.x < mval) && (_PuckVel.x > -mval) && (_PuckPos.x <= 0) && (_PuckPos.x > m_Position.x))
		{ 
			m_Velocity.x = m_MaxVelocity.x; 
		}
		break;

	default:
		break;
	}
	
	Move(float(x), float(z));
}

void CPlayer::Move(float x, float z)
{
	m_Acceleration.Set(m_MaxAcceleration.x * x, 0, m_MaxAcceleration.z * z);
//	m_Velocity += accel;
	
	if(m_Velocity.x >  m_MaxVelocity.x)	m_Velocity.x =  m_MaxVelocity.x;
	if(m_Velocity.x < -m_MaxVelocity.x) m_Velocity.x = -m_MaxVelocity.x;
	if(m_Velocity.z >  m_MaxVelocity.z)	m_Velocity.z =  m_MaxVelocity.z;
	if(m_Velocity.z < -m_MaxVelocity.z) m_Velocity.z = -m_MaxVelocity.z;

}

void CPlayer::MMove(float x, float z)
{
	vec3f accel(x, 0, z);
	m_Velocity = accel;
}

bool CPlayer::Thrust(float _magnitude)
{	
	switch(m_BaseAxis)
	{
	case X:		m_Velocity.x = _magnitude * m_MaxVelocity.x;	break;
	case Z:		m_Velocity.z = _magnitude * m_MaxVelocity.z;	break;
	}
	return true;
}

int  CPlayer::Update( void* _arg)
{ 

	double tp = *(double*) _arg;

	float dzone = 0.1f;

	switch(m_BaseAxis)
	{
	case X:		
		if(m_Position.x > m_BaseLine+dzone) 
			m_Acceleration.x -= m_MaxAcceleration.x;
		if(m_Position.x < m_BaseLine-dzone) 
			m_Acceleration.x += m_MaxAcceleration.x;
		break;
	}	

	m_Velocity += m_Acceleration * float(tp);

	m_Velocity *= m_SurfaceFriction;
	m_Position += m_Velocity * float(tp);

	vec3f lsize = g_WorldController.m_Level->m_Size * 0.5f;
	if(m_Position.x >  lsize.x - m_Scale.x * 0.5f) 
	{
		m_Position.x = lsize.x - m_Scale.x * 0.5f;
		if(m_Velocity.x > 0) 
			m_Velocity.x *= -0.9f;
	}
	if(m_Position.x < -lsize.x + m_Scale.x * 0.5f)
	{
		m_Position.x = -lsize.x + m_Scale.x * 0.5f;
		if(m_Velocity.x < 0)
			m_Velocity.x *= -0.9f;
	}
	
	if(m_Position.z >  lsize.z - m_Scale.x * 0.5f)
	{
		m_Position.z =  lsize.z - m_Scale.x * 0.5f;
		if(m_Velocity.z > 0)
			m_Velocity.z *= -0.9f;
	}
	if(m_Position.z < -lsize.z + m_Scale.x * 0.5f)
	{
		m_Position.z = -lsize.z + m_Scale.x * 0.5f;
		if(m_Velocity.z < 0)
			m_Velocity.z *= -0.9f;
	}

	void *xm = 0;
	m_Trail.SetPos(m_Position);
	//m_Trail.UpdateFlat(m_Velocity);
	m_Trail.Update(_arg);

	return 1; 
}

bool CPlayer::Destroy()	
{
	m_Trail.Destroy();
	return 1;
}

void CPlayer::Render(bool _doTrail)
{

	if(_doTrail) m_Trail.Render();

	glPushMatrix();
	glColor4f(m_BaseColour.r, m_BaseColour.g, m_BaseColour.b, m_BaseColour.a);
	glTranslatef (m_Position.x, m_Position.y, m_Position.z);
	g_Renderer.RenderCylinder(m_Scale.x*0.5f, 16, 0.1f);
	glPopMatrix();	

}