#include "../Globals.h"
#include "Puck.h"

bool CPuck::Init()			
{ 
	m_Position.Set(0,0,0);
	m_Rotation.Set(0,0,0);
	m_Scale.Set(0.5f, 0.5f, 0.5f);
	m_BaseColour.Set(0,0,0,1);
	m_Trail.Init(1, 10, 0, m_Position, m_BaseColour, m_Scale*0.5);

	return 1; 
}

bool CPuck::Init(vec3f &_pos, col4f &_col)			
{ 
	m_Position.Set(_pos);
	m_Rotation.Set(0,0,0);
	m_Scale.Set(0.5f, 0.5f, 0.5f);
	m_BaseColour.Set(&_col);
	m_Velocity.Set(0,0,0);	
	m_MaxVelocity.Set(10.075f, 10.075f, 10.075f);
	
	col4f b = m_BaseColour;
	b.a = 0.75;
	m_Trail.Init(20, 100, 0, m_Position, b, m_Scale*0.5);

	return 1; 
}

void CPuck::Move(float x, float z)
{
//	vec3f accel(m_Acceleration.x * x, 0, m_Acceleration.z * z);
//	m_Velocity += accel;
	
	if(m_Velocity.x >  m_MaxVelocity.x)	m_Velocity.x =  m_MaxVelocity.x;
	if(m_Velocity.x < -m_MaxVelocity.x) m_Velocity.x = -m_MaxVelocity.x;
	if(m_Velocity.z >  m_MaxVelocity.z)	m_Velocity.z =  m_MaxVelocity.z;
	if(m_Velocity.z < -m_MaxVelocity.z) m_Velocity.z = -m_MaxVelocity.z;

}

int  CPuck::Update( void* _arg )
{ 

	double tp = *(double*) _arg;

	m_Velocity *= 0.9999f;
	m_Position += m_Velocity * float(tp);
	float absorb = -0.9f;

	vec3f lsize = g_WorldController.m_Level->m_Size * 0.5f;
	float g1 = g_WorldController.m_Level->m_1UpGoalWidth;
	float g2 = g_WorldController.m_Level->m_1UpGoalWidth;
	
	if((m_Position.z >  lsize.z - m_Scale.z * 0.5f) && m_Velocity.z > 0) m_Velocity.z *= absorb;
	if((m_Position.z < -lsize.z + m_Scale.z * 0.5f) && m_Velocity.z < 0) m_Velocity.z *= absorb;

	if((m_Position.x >  lsize.x - m_Scale.x * 0.5f) && m_Velocity.x > 0) m_Velocity.x *= absorb;
	if((m_Position.x < -lsize.x + m_Scale.x * 0.5f) && m_Velocity.x < 0) m_Velocity.x *= absorb;

	m_Trail.m_Position.Set(m_Position);
	m_Trail.UpdateFlat(m_Velocity);

	return 1; 
}

bool CPuck::Destroy()	
{
	m_Trail.Destroy();
	return 1;
}

void CPuck::Render(bool _doTrail)
{
	if(_doTrail) m_Trail.Render();

	glPushMatrix();
	glColor4f(m_BaseColour.r, m_BaseColour.g, m_BaseColour.b, m_BaseColour.a);
	glTranslatef (m_Position.x, m_Position.y, m_Position.z);
	g_Renderer.RenderCylinder(m_Scale.x*0.5f, 16, 0.1f);
	glPopMatrix();	


}