//=========================================================================
// 
//-------------------------------------------------------------------------
//
// This is where it all happens
//
//=========================================================================

#include "Core/GLCore.h"	
#include "Maths/maths.h"	
#include "Render/Render.h"
#include "Globals.h"

//GL_Window*	g_window;
Keys*		g_keys;
Mouse*		g_mouse;

float		g_Light_Ambient[4]	= { 0.0f, 0.0f, 0.0f, 1.0f };
float		g_Light_Diffuse[4]	= { 1.0f, 1.0f, 1.0f, 1.0f };
float		g_Light_Position[4]	= { 5.0f, 15.0f, 5.0f, 1.0f };


// User Defined Variables
float		angle;												// Used To Rotate The Triangles
int			rot1, rot2;											// Counter Variables

BOOL Initialize (GL_Window* window, Keys* keys, Mouse* mouse)					// Any GL Init Code & User Initialiazation Goes Here
{
	g_Renderer.m_window = window;
	g_Renderer.m_ReflectEnabled = true;
	g_keys		= keys;
	g_mouse		= mouse;
	g_Camera.SetCamera(	-15,	8,		0,
						-2.5,	0,		0,
						0,		1,		0);

	// Start Of User Initialization
	angle		= 0.0f;											// Set Starting Angle To Zero

	glClearColor (0.9f, 0.9f, 0.9f, 0.5f);						// Black Background
	glClearDepth (1.0f);										// Depth Buffer Setup
	glDepthFunc (GL_LEQUAL);									// The Type Of Depth Testing (Less Or Equal)
	glEnable (GL_DEPTH_TEST);									// Enable Depth Testing
	glShadeModel (GL_SMOOTH);									// Select Smooth Shading
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate

	glLightfv(GL_LIGHT1, GL_AMBIENT, g_Light_Ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, g_Light_Diffuse);
	glLightfv(GL_LIGHT1, GL_POSITION,g_Light_Position);
	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);

	glEnable(GL_LIGHT1);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);

	glPolygonMode( GL_FRONT, GL_FILL );
	glPolygonMode( GL_BACK, GL_FILL );
	glEnable( GL_CULL_FACE );
	glFrontFace( GL_CCW );

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	//glEnable(GL_TEXTURE_2D);

	g_Renderer.m_window->BuildFont();

	g_Renderer.m_PTex = g_Renderer.m_TextureMan.LoadTextureTGA("Textures/Point.tga");


	g_Player1.Init();
	g_Player2.Init();
	g_WorldController.Init(&g_Player1, &g_Player2, &g_Level);

	return TRUE;												// Return TRUE (Initialization Successful)
}

void Deinitialize (void)										// Any User DeInitialization Goes Here
{

}

void LocalInput()
{

}

int  Update (DWORD milliseconds)								// Perform Motion Updates Here
{
	if (g_keys->keyDown [VK_ESCAPE] == TRUE){	TerminateApplication (g_Renderer.m_window);	}
	if (g_keys->keyDown [VK_F1] == TRUE)	{	ToggleFullscreen (g_Renderer.m_window);		}
	if (g_keys->keyDown [VK_F2] == TRUE)	{	g_Renderer.m_ReflectEnabled = true;			}	
	if (g_keys->keyDown [VK_F3] == TRUE)	{	g_Renderer.m_ReflectEnabled = false;		}	

	if (g_keys->toggled ['1'] == TRUE)		{	Toggle(g_WorldController.m_1UP_AI);	}
	if (g_keys->toggled ['2'] == TRUE)		{	Toggle(g_WorldController.m_2UP_AI);	}


	int x = 0, z = 0;
	if (g_keys->keyDown [VK_UP] == TRUE)	{	g_WorldController.m_Player1->Thrust(1);	  }
	if (g_keys->keyDown [VK_DOWN] == TRUE)	{	x = -1; }
	if (g_keys->keyDown [VK_LEFT] == TRUE)	{	z = -1;	}
	if (g_keys->keyDown [VK_RIGHT] == TRUE)	{	z = 1;	}
	g_WorldController.m_Player1->Move(float(x), float(z));

	// Level shape
	if (g_keys->keyDown ['W'] == TRUE)		
	{	
			g_WorldController.m_Level->m_Size.x += 0.5f; 
			g_WorldController.m_Player1->m_BaseLine = -0.5f * g_WorldController.m_Level->m_Size.x + g_WorldController.m_Player1->m_Scale.x *1.5f;
			g_WorldController.m_Player2->m_BaseLine =  0.5f * g_WorldController.m_Level->m_Size.x - g_WorldController.m_Player1->m_Scale.x *1.5f;
	}
	if (g_keys->keyDown ['S'] == TRUE)		
	{	
		if(g_WorldController.m_Level->m_Size.x > (g_WorldController.m_Player1->m_Scale.x * 1.5f + g_WorldController.m_Player2->m_Scale.x * 1.5f ))
		{
			g_WorldController.m_Level->m_Size.x -= 0.5f; 
			g_WorldController.m_Player1->m_BaseLine = -0.5f * g_WorldController.m_Level->m_Size.x + g_WorldController.m_Player1->m_Scale.x *1.5f;
			g_WorldController.m_Player2->m_BaseLine =  0.5f * g_WorldController.m_Level->m_Size.x - g_WorldController.m_Player1->m_Scale.x *1.5f;
		}
	}
	if (g_keys->keyDown ['D'] == TRUE)
	{	
		g_WorldController.m_Level->m_Size.z += 0.5f; 
	}
	if (g_keys->keyDown ['A'] == TRUE)		
	{	
		if(g_WorldController.m_Level->m_Size.z > (g_WorldController.m_Player1->m_Scale.x * 1.5f + g_WorldController.m_Player2->m_Scale.x * 1.5f ))
		{
			g_WorldController.m_Level->m_Size.z -= 0.5f; 
		}

	}

	//Other shapes
	if (g_keys->keyDown ['H'] == TRUE)	{	g_WorldController.m_Level->m_1UpGoalWidth += 0.125f; }
	if (g_keys->keyDown ['G'] == TRUE)	{	g_WorldController.m_Level->m_1UpGoalWidth -= 0.125f; }
	if (g_keys->keyDown ['Y'] == TRUE)	{	g_WorldController.m_Level->m_2UpGoalWidth += 0.125f; }
	if (g_keys->keyDown ['T'] == TRUE)	{	g_WorldController.m_Level->m_2UpGoalWidth -= 0.125f; }

	//Camera
	float mag = 0.05f;
	float xm = g_mouse->XMove * mag;
	float ym = -g_mouse->YMove * mag;
	if(g_mouse->Left)
	{
		g_Camera.RotateYX(ym, xm, 1);
	}
	if(g_mouse->Right)
	{
		g_Camera.Advance(ym);
		g_Camera.Strafe(xm);
	}
	if(g_mouse->Middle)
	{
		g_Camera.Ascend(ym);
		g_Camera.Strafe(xm);
	}

	if (g_keys->keyDown ['P'] == TRUE)	
	{
		g_FXManager.AddParticleSys("", vec3f(0,0,0));
	}


	g_WorldController.Update();
	angle += (float)(milliseconds) * 0.05f;						// Update angle Based On The Clock

	return 1;
}

void Draw (void)
{	
	StartFrame();

	g_Renderer.SetCamera(&g_Camera);

	glLightfv(GL_LIGHT1, GL_AMBIENT, g_Light_Ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, g_Light_Diffuse);
	glLightfv(GL_LIGHT1, GL_POSITION,g_Light_Position);

	g_WorldController.Render();

	// Draw text
	glLoadIdentity();									// Reset The Current Modelview Matrix
	glTranslatef(0.0f,0.0f,-1.0f);						// Move One Unit Into The Screen
	glColor3f(0,0,0);
	glRasterPos2f(-0.525f, 0.375f);
	g_Renderer.m_window->glPrint("%d | %d (%2.0f)", g_WorldController.m_Player1->m_Score, g_WorldController.m_Player2->m_Score, g_WorldController.m_FPS);
	EndFrame();
}
