#include "maths.h"


void frustumf::ExtractPlanesGL( mat4f * _matrix, bool _normalize)
{
	m_left.m_equ.x = _matrix->cell[3][0] + _matrix->cell[0][0];
	m_left.m_equ.y = _matrix->cell[3][1] + _matrix->cell[0][1];
	m_left.m_equ.z = _matrix->cell[3][2] + _matrix->cell[0][2];
	m_left.m_equ.a = _matrix->cell[3][3] + _matrix->cell[0][3];
	// Right clipping plane
	m_right.m_equ.x = _matrix->cell[3][0] - _matrix->cell[0][0];
	m_right.m_equ.y = _matrix->cell[3][1] - _matrix->cell[0][1];
	m_right.m_equ.z = _matrix->cell[3][2] - _matrix->cell[0][2];
	m_right.m_equ.a = _matrix->cell[3][3] - _matrix->cell[0][3];
	// Top clipping plane
	m_up.m_equ.x = _matrix->cell[3][0] - _matrix->cell[1][0];
	m_up.m_equ.y = _matrix->cell[3][1] - _matrix->cell[1][1];
	m_up.m_equ.z = _matrix->cell[3][2] - _matrix->cell[1][2];
	m_up.m_equ.a = _matrix->cell[3][3] - _matrix->cell[1][3];
	// Bottom clipping plane
	m_down.m_equ.x = _matrix->cell[3][0] + _matrix->cell[1][0];
	m_down.m_equ.y = _matrix->cell[3][1] + _matrix->cell[1][1];
	m_down.m_equ.z = _matrix->cell[3][2] + _matrix->cell[1][2];
	m_down.m_equ.a = _matrix->cell[3][3] + _matrix->cell[1][3];
	// Near clipping plane
	m_near.m_equ.x = _matrix->cell[3][0] + _matrix->cell[2][0];
	m_near.m_equ.y = _matrix->cell[3][1] + _matrix->cell[2][1];
	m_near.m_equ.z = _matrix->cell[3][2] + _matrix->cell[2][2];
	m_near.m_equ.a = _matrix->cell[3][3] + _matrix->cell[2][3];
	// Far clipping plane
	m_far.m_equ.x = _matrix->cell[3][0] - _matrix->cell[2][0];
	m_far.m_equ.y = _matrix->cell[3][1] - _matrix->cell[2][1];
	m_far.m_equ.z = _matrix->cell[3][2] - _matrix->cell[2][2];
	m_far.m_equ.a = _matrix->cell[3][3] - _matrix->cell[2][3];
	// Normalize the plane equations, if requested
	if (_normalize)
	{
		m_near.Normalise();
		m_far.Normalise();
		m_up.Normalise();
		m_down.Normalise();
		m_left.Normalise();
		m_right.Normalise();
	}
}



bool frustumf::ContainsAABB( AABB * _box )
{
	vec3f _corners[8];

	// get the corners of the box into the vCorner array
	_box->GetVertices( _corners );


	// check if outside any planes
	if (m_near.BoxTotNegCheck( _corners )) return false;
	if (m_far.BoxTotNegCheck( _corners )) return false;
	if (m_left.BoxTotNegCheck( _corners )) return false;
	if (m_right.BoxTotNegCheck( _corners )) return false;
	if (m_up.BoxTotNegCheck( _corners )) return false;
	if (m_down.BoxTotNegCheck( _corners )) return false;

	// otherwise return true
	return true;
}
