#include "maths.h"

// mat3f.cell[column][row]

mat3f mat3f::operator+ (mat3f param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] + param.cell[i][j];

	return (temp);
}

mat3f mat3f::operator+= (mat3f param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] += param.cell[i][j];
	
	return (temp);
}

mat3f mat3f::operator- (mat3f param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] - param.cell[i][j];

	return (temp);
}

mat3f mat3f::operator-= (mat3f param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] -= param.cell[i][j];
	
	return (temp);
}

mat3f mat3f::operator* (mat3f param)
{
	mat3f temp;

	int i,j,k;
	float sum;

	for (i =0; i<3; i++){
		for (j=0; j<3; j++){
			sum = 0; 
			for (k=0; k<3; k++){
				sum += cell[k][i] * param.cell[j][k];
			}
			temp.cell[j][i] = sum;
		}
	}

	return (temp);
}

mat3f mat3f::operator*= (mat3f param)
{
	mat3f temp;

	int i,j,k;
	float sum;

	for (i =0; i<3; i++){
		for (j=0; j<3; j++){
			sum = 0; 
			for (k=0; k<3; k++){
				sum += cell[k][i] * param.cell[j][k];
			}
			temp.cell[j][i] = sum;
		}
	}

	*this = temp;
	return (temp);
}

vec3f mat3f::operator * (vec3f param)
{
	vec3f temp;

	temp.x = (cell[0][0] * param.x) + (cell[0][1] * param.y) + (cell[0][2] * param.z);
	temp.y = (cell[1][0] * param.x) + (cell[1][1] * param.y) + (cell[1][2] * param.z);
	temp.z = (cell[2][0] * param.x) + (cell[2][1] * param.y) + (cell[2][2] * param.z);

	return (temp);
}

mat3f mat3f::operator * (float param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] * param;

	return (temp);
}


mat3f mat3f::operator*= (float param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] *= param;
	
	return (temp);
}

mat3f mat3f::operator/ (float param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] / param;

	return (temp);
}

mat3f mat3f::operator/= (float param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] /= param;
	
	return (temp);
}

mat3f mat3f::operator= (float param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = param;
	
	return (temp);
}

mat3f mat3f::operator= (mat3f param)
{
	mat3f temp;

	int i, j;
	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			temp.cell[i][j] = cell[i][j] = param.cell[i][j];
	
	return (temp);
}

bool mat3f::operator== (mat3f param)
{
	int i,j;

	for (i=0; i<3; i++)
		for (j=0; j<3; j++)
			if (fabs(param.cell[i][j] - cell[i][j]) > EQUALENOUGH)
			{
				return 0;
			}

	return 1;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

mat3f::mat3f( void )
{
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			if (i==j) cell[i][j] = 1.0f;
			else cell[i][j] = 0.0f;
}

mat3f::mat3f( quatf * _param )
{
	*this = QToM3( _param );
}

mat3f::mat3f( vec3f * _C1, vec3f * _C2, vec3f * _C3 )
{
	cell[0][0] = _C1->x;
	cell[0][1] = _C1->y;
	cell[0][2] = _C1->z;
	cell[1][0] = _C2->x;
	cell[1][1] = _C2->y;
	cell[1][2] = _C2->z;
	cell[2][0] = _C3->x;
	cell[2][1] = _C3->y;
	cell[2][2] = _C3->z;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

float mat3f::Det( void )
{
	float temp;

	temp = (cell[0][0] * (( cell[1][1] * cell[2][2] ) - ( cell[1][2] * cell[2][1] )))
		 - (cell[0][1] * (( cell[1][0] * cell[2][2] ) - ( cell[1][2] * cell[2][0] )))
		 + (cell[0][2] * (( cell[1][0] * cell[2][1] ) - ( cell[1][1] * cell[2][0] )));

	return (temp);
}


void mat3f::Set( int column, float n1, float n2, float n3 )
{
	cell[column][0] = n1;
	cell[column][1] = n2;
	cell[column][2] = n3;
}


mat3f mat3f::Inverse( void )
{
	mat3f temp;

	temp.cell[0][0] = (cell[1][1] * cell[2][2]) - (cell[1][2] * cell[2][1]);
	temp.cell[0][1] = (cell[0][2] * cell[2][1]) - (cell[0][1] * cell[2][2]);
	temp.cell[0][2] = (cell[0][1] * cell[1][2]) - (cell[0][2] * cell[1][1]);

	temp.cell[1][0] = (cell[1][2] * cell[2][0]) - (cell[1][0] * cell[2][2]);
	temp.cell[1][1] = (cell[0][0] * cell[2][2]) - (cell[0][2] * cell[2][0]);
	temp.cell[1][2] = (cell[0][2] * cell[1][0]) - (cell[0][0] * cell[1][2]);

	temp.cell[2][0] = (cell[1][0] * cell[2][1]) - (cell[1][1] * cell[2][0]);
	temp.cell[2][1] = (cell[0][1] * cell[2][0]) - (cell[0][0] * cell[2][1]);
	temp.cell[2][2] = (cell[0][0] * cell[1][1]) - (cell[0][1] * cell[1][0]);

	temp /= Det();

	return (temp);
}


mat3f mat3f::Transpose( void )
{
	mat3f temp;
	int i,j;

	for ( i=0; i<3; i++)
		for ( j=0; j<3; j++)
			temp.cell[i][j] = cell[j][i];

	return (temp);
}
