#include "maths.h"




quatf M4ToQ( mat4f * _param )
{
	quatf temp;

	float fTr = _param->cell[0][0] + _param->cell[1][1] + _param->cell[2][2] + 1.0f;

	//if the trace of the matrix is greater than 0, perform an instant calculation
	if(fTr > 0)
	{
		float fS = 0.5f / sqrtf(fTr);

		temp.w = 0.25f / fS;
		temp.x = (_param->cell[1][2] - _param->cell[2][1]) * fS;
		temp.y = (_param->cell[2][0] - _param->cell[0][2]) * fS;
		temp.z = (_param->cell[0][1] - _param->cell[1][0]) * fS;
	}

	//if the first diagonal element is the largest
	else if(_param->cell[0][0] > _param->cell[1][1] && _param->cell[0][0] > _param->cell[2][2])
	{
		float fS = 1 / ((float)sqrt(_param->cell[0][0] - _param->cell[1][1] - _param->cell[2][2] + 1.0f) * 2);
		temp.x = 0.5f * fS;
		temp.y = (_param->cell[1][0] + _param->cell[0][1]) * fS;
		temp.z = (_param->cell[2][0] + _param->cell[0][2]) * fS;
		temp.w = (_param->cell[2][1] + _param->cell[1][2]) * fS;
	}

	//if the second.....
	else if(_param->cell[1][1] > _param->cell[2][2])
	{
		float fS = 1 / ((float)sqrt(_param->cell[1][1] - _param->cell[0][0] - _param->cell[2][2] + 1.0f) * 2);
		temp.x = (_param->cell[1][0] + _param->cell[0][1]) * fS;
		temp.y = 0.5f * fS;
		temp.z = (_param->cell[2][1] + _param->cell[1][2]) * fS;
		temp.w = (_param->cell[2][0] + _param->cell[0][2]) * fS;
	}

	//if the last element is the largest
	else
	{
		float fS = 1 / ((float)sqrt(_param->cell[2][2] - _param->cell[0][0] - _param->cell[1][1] + 1.0f) * 2);
		temp.x = (_param->cell[2][0] + _param->cell[0][2]) * fS;
		temp.y = (_param->cell[2][1] + _param->cell[1][2]) * fS;
		temp.z = 0.5f * fS;
		temp.w = (_param->cell[1][0] + _param->cell[0][1]) * fS;
	}

	return temp;
}




mat4f QToM4( quatf * _param)
{
	mat4f temp;

	temp.cell[0][0] = 1 - (2 * _param->y * _param->y) - (2 * _param->z * _param->z);
	temp.cell[1][0] = (2 * _param->x * _param->y) - (2 * _param->w * _param->z);
	temp.cell[2][0] = (2 * _param->x * _param->z) + (2 * _param->w * _param->y);
	temp.cell[3][0] = 0;

	temp.cell[0][1] = (2 * _param->x * _param->y) + (2 * _param->w * _param->z);
	temp.cell[1][1] = 1 - (2 * _param->x * _param->x) - (2 * _param->z * _param->z);
	temp.cell[2][1] = (2 * _param->y * _param->z) - (2 * _param->w * _param->x);
	temp.cell[3][1] = 0;

	temp.cell[0][2] = (2 * _param->x * _param->z) - (2 * _param->w * _param->y);
	temp.cell[1][2] = (2 * _param->y * _param->z) + (2 * _param->w * _param->x);
	temp.cell[2][2] = 1 - (2 * _param->x * _param->x) - (2 * _param->y * _param->y);
	temp.cell[3][2] = 0;

	temp.cell[0][3] = 0;
	temp.cell[1][3] = 0;
	temp.cell[2][3] = 0;
	temp.cell[3][3] = 1;

	return (temp);
}


quatf M3ToQ( mat3f * _param )
{
	quatf temp;

	float fTr = _param->cell[0][0] + _param->cell[1][1] + _param->cell[2][2] + 1.0f;

	//if the trace of the matrix is greater than 0, perform an instant calculation
	if(fTr > 0)
	{
		float fS = 0.5f / sqrtf(fTr);

		temp.w = 0.25f / fS;
		temp.x = (_param->cell[1][2] - _param->cell[2][1]) * fS;
		temp.y = (_param->cell[2][0] - _param->cell[0][2]) * fS;
		temp.z = (_param->cell[0][1] - _param->cell[1][0]) * fS;
	}

	//if the first diagonal element is the largest
	else if(_param->cell[0][0] > _param->cell[1][1] && _param->cell[0][0] > _param->cell[2][2])
	{
		float fS = 1 / ((float)sqrt(_param->cell[0][0] - _param->cell[1][1] - _param->cell[2][2] + 1.0f) * 2);
		temp.x = 0.5f * fS;
		temp.y = (_param->cell[1][0] + _param->cell[0][1]) * fS;
		temp.z = (_param->cell[2][0] + _param->cell[0][2]) * fS;
		temp.w = (_param->cell[2][1] + _param->cell[1][2]) * fS;
	}

	//if the second.....
	else if(_param->cell[1][1] > _param->cell[2][2])
	{
		float fS = 1 / ((float)sqrt(_param->cell[1][1] - _param->cell[0][0] - _param->cell[2][2] + 1.0f) * 2);
		temp.x = (_param->cell[1][0] + _param->cell[0][1]) * fS;
		temp.y = 0.5f * fS;
		temp.z = (_param->cell[2][1] + _param->cell[1][2]) * fS;
		temp.w = (_param->cell[2][0] + _param->cell[0][2]) * fS;
	}

	//if the last element is the largest
	else
	{
		float fS = 1 / ((float)sqrt(_param->cell[2][2] - _param->cell[0][0] - _param->cell[1][1] + 1.0f) * 2);
		temp.x = (_param->cell[2][0] + _param->cell[0][2]) * fS;
		temp.y = (_param->cell[2][1] + _param->cell[1][2]) * fS;
		temp.z = 0.5f * fS;
		temp.w = (_param->cell[1][0] + _param->cell[0][1]) * fS;
	}

	return temp;
}




mat3f QToM3( quatf * _param)
{
	mat3f temp;

	temp.cell[0][0] = 1 - (2 * _param->y * _param->y) - (2 * _param->z * _param->z);
	temp.cell[1][0] = (2 * _param->x * _param->y) - (2 * _param->w * _param->z);
	temp.cell[2][0] = (2 * _param->x * _param->z) + (2 * _param->w * _param->y);

	temp.cell[0][1] = (2 * _param->x * _param->y) + (2 * _param->w * _param->z);
	temp.cell[1][1] = 1 - (2 * _param->x * _param->x) - (2 * _param->z * _param->z);
	temp.cell[2][1] = (2 * _param->y * _param->z) - (2 * _param->w * _param->x);

	temp.cell[0][2] = (2 * _param->x * _param->z) - (2 * _param->w * _param->y);
	temp.cell[1][2] = (2 * _param->y * _param->z) + (2 * _param->w * _param->x);
	temp.cell[2][2] = 1 - (2 * _param->x * _param->x) - (2 * _param->y * _param->y);

	return (temp);
}




vec3f planeNormal(vec3f &_p1, vec3f &_p2, vec3f &_p3)
{
	vec3f tempa,tempb,tempc;
	
	tempa=_p1-_p2;
	tempb=_p2-_p3;
	
	tempc = tempa.CrossProduct(tempb);
	tempc.Normalize();

	return tempc;
}



col4f LightingForVertex(vec3f &_vpos, vec3f &_vnorm, vec3f &_lpos, vec3f &_latten, col4f &_lcol, float _scaleFactor)
{
	float i, d, l;						//intensity, dist, length
	vec3f lv, n;						//light vector and normal

	lv = _lpos - _vpos;
	l  = _scaleFactor * float(fabs(lv.Length()));

	lv.Normalize();

	n  = _vnorm;
	i  = n.DotProduct(lv);
	
	d = 1 / ( _latten.x + 
			( _latten.y * l ) + 
			( _latten.z * l * l ) );


	if(d>1)d=1;							//cap colours at 1.0f
	
	col4f c = _lcol;
	c *= (i * d);
	
	return c;
	
}


