#ifndef _MATHS_H_
#define _MATHS_H_

//////////////////////////////
//	#defines - constants	//
//////////////////////////////
#define EQUALENOUGH 0.0001  

#define PI					(3.14159f)
#define PIOVER180			(0.017453277777777777777777777777778f)		//(PI/180.0f)
#define PIUNDER180			(57.295827908797774375395898255342f)		//(180.0f/PI)
#define ONEOVERRANDMAX		(0.000030518509475997192297128208258309f)

//////////////////
//	#includes	//
//////////////////
#include <math.h>
#include <stdlib.h>
#include <time.h>

class vec2f;
class vec3f;
class vec4f;
class mat3f;
class mat4f;
class quatf;
class planef;
class frustumf;
class AABB;

#include "vec2f.h"
#include "vec3f.h"
#include "vec4f.h"
#include "mat3f.h"
#include "mat4f.h"
#include "quatf.h"
#include "planef.h"
//#include "../collisions/collisions.h"
//#include "../octrees/octrees.h"
//#include "frustumf.h"

#include "colDefault.h"

//////////////////////////
//	#defines - macros	//
//////////////////////////
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

#define SQR(x) ((x)*(x))
#define SQRT(x) ((float)sqrt(x))

#define SIN(x) ((float)sin(x))
#define COS(x) ((float)cos(x))

//////////////////
//	functions	//
//////////////////
quatf M4ToQ( mat4f * param );
mat4f QToM4( quatf * param );
quatf M3ToQ( mat3f * param );
mat3f QToM3( quatf * param );

vec3f planeNormal(vec3f &_p1, vec3f &_p2, vec3f &_p3);
col4f LightingForVertex(vec3f &_vpos, vec3f &_vnorm, vec3f &_lpos, vec3f &_latten, col4f &_lcol, float _scaleFactor);
// Shamelessly used from Miguel Gomez's article ( http://www.gamasutra.com/features/19991018/Gomez_2.htm )
// Return true if r1 and r2 are real, and put values of roots in r1 & r2
inline bool QuadraticFormula(const float a, const float b, const float c, float& r1, float& r2)
{
	const float q = b*b - 4*a*c;
    if( q > 0 )
    {
        const float sq = (float)sqrt(q);
        const float d = 1 / (2*a);
        r1 = ( -b + sq ) * d;
        r2 = ( -b - sq ) * d;
        return true;					//real roots
    }
    else
    {
		return false;					//complex roots
    }

}
template< class T >
inline void SWAP( T& a, T& b )
{
    const T temp = a;
    a = b;
    b = temp;
}

//////////////
//	inlines	//
//////////////

// random
	// The following line must be at the start of the main function
	// srand(time(NULL));
	inline float random( void )
	{
		return ((float)rand())*ONEOVERRANDMAX;
	}


#endif