#include "UDPSocket.h"

#include <stdio.h>
#pragma comment (lib, "ws2_32.lib")	// Make sure the winsock library is included
#include <assert.h>

C_UDPSocket::C_UDPSocket()
{
	m_SocketAddressSize = sizeof(sockaddr_in);
	m_Socket = 0;
}

C_UDPSocket::~C_UDPSocket()
{
	if(m_Socket)
		closesocket(m_Socket);

	WSACleanup ();	
	m_Socket = 0;
}

int C_UDPSocket::Init()
{
	// Widows specific setup.
	int error = WSAStartup (0x0202,&m_WSData);
	if (error)
	{
		assert("You need WinSock 2.2\n");
		return 0;
	}
	if (m_WSData.wVersion!=0x0202)
	{
		assert("Error: Wrong WinSock version!\n");
		WSACleanup ();
		return 0;
	}

	
	// socket creation
	m_Socket = socket(AF_INET, SOCK_DGRAM, 0);
	if(m_Socket<0) 
	{
		assert("Cannot open socket \n");
		return 0;
	}
	
	return 1;
}

int C_UDPSocket::SetNonBlocking()
{
	u_long NonBlocking = TRUE;
	if ((ioctlsocket(m_Socket, FIONBIO, (u_long*)&NonBlocking)) == SOCKET_ERROR)
	{
		assert("Error: Can't make Socket nonblocking\n");
		return 0;
	}

	return 1;
}

int C_UDPSocket::Bind(const int _port)
{

	// Bind socket to local machine
	m_LocalAddress.sin_family 		= AF_INET;
	m_LocalAddress.sin_addr.s_addr 	= htonl(INADDR_ANY);
	m_LocalAddress.sin_port 		= htons(_port);
	
	int rc = bind (m_Socket, (struct sockaddr *)&m_LocalAddress, m_SocketAddressSize);
	if(rc<0) 
	{
		char x[256];
		sprintf(x, "cannot bind port number %d \n", _port);
		assert(x);
		return 0;
	}

	return 1;	
}

int C_UDPSocket::Send(char *_buffer)
{
	int n = sendto(	m_Socket, 
					_buffer, 
					PACKET_SIZE, 
					0, 
					(struct sockaddr *)&m_RemoteAddress, 
					m_SocketAddressSize
					);

	return n;	
}

int C_UDPSocket::Recieve(char *_buffer)
{
	int n =recvfrom(m_Socket, 
					_buffer, 
					PACKET_SIZE, 
					0, 
					(struct sockaddr *)&m_RemoteAddress, 
					&m_SocketAddressSize
					);	
	return n;	
}


void C_UDPSocket::SetDestAddress(char *_ip, const int _port)
{
	m_RemoteAddress.sin_family = AF_INET;
	m_RemoteAddress.sin_port = htons (_port);
	m_RemoteAddress.sin_addr.s_addr = inet_addr (_ip);
}

sockaddr_in C_UDPSocket::GetDestAddress(void)
{
	return m_RemoteAddress;
}


