#include "Render.h"

void CRenderer::RenderLine(vec3f &_start, vec3f &_end, float _width)
{
	glLineWidth(_width);
	glBegin( GL_LINES );
	{
		glVertex3f(_start.x, _start.y, _start.z);
		glVertex3f(_end.x, _end.y, _end.z);
	}
	glEnd();

}

void CRenderer::RenderCircle(float _radius, float _resolution, float _lineWidth)
{
		float step, at=0;	

		glLineWidth(_lineWidth);

		step = _resolution * PIOVER180;
		glBegin(GL_LINE_LOOP);
		{
			while(at < 2 * PI)
			{
				glVertex3f(float(_radius * sin(at)), 0.0f, float(_radius * cos(at)));
				at+=step;
			}
		}
		glEnd();

		glLineWidth(1);
}

void CRenderer::RenderDisc(float _radius, float _resolution)
{
		float step, at=0;	

		_resolution = 360 / _resolution;
		step = _resolution * PIOVER180;
		glBegin(GL_TRIANGLE_FAN);
		{
			glNormal3f( 0.0f, 1.0f, 0.0f);
			glVertex3f(0,0,0);
			while(at < 2 * PI)
			{
				glVertex3f(float(_radius * sin(at)), 0.0f, float(_radius * cos(at)));
				at+=step;
			}
		}
		glEnd();
}

void CRenderer::RenderCylinder(float _radius, float _resolution, float height)
{
		float step, at=2*PI;	

		_resolution = 360 / _resolution;
		step = _resolution * PIOVER180;
		glBegin(GL_TRIANGLE_STRIP);
		{
			while(at > 0)
			{
				glNormal3f(float(sin(at)),				0.0f,	float(cos(at)));
				glVertex3f(float(_radius * sin(at)),	0.0f,	float(_radius * cos(at)));
				glVertex3f(float(_radius * sin(at)),	height, float(_radius * cos(at)));
				at-=step;
			}
		}
		glEnd();
		
		glBegin(GL_TRIANGLE_FAN);
		{
			glNormal3f( 0.0f, 1.0f, 0.0f);
			glVertex3f(0,0,0);
			while(at < 2 * PI)
			{
				glVertex3f(float(_radius * sin(at)),	height,	float(_radius * cos(at)));
				at+=step;
			}
		}
		glEnd();


}

void CRenderer::RenderAxes(float _scale, float _width, float _alpha)
{
	vec3f O(0,0,0);
	vec3f X(1,0,0);
	vec3f Y(0,1,0);
	vec3f Z(0,0,1);

	glColor4f(1.0f, 0.0f, 0.0f, _alpha);
	RenderLine(O,X,_width);
	glColor4f(0.0f, 1.0f, 0.0f, _alpha);
	RenderLine(O,Y,_width);
	glColor4f(0.0f, 0.0f, 1.0f, _alpha);
	RenderLine(O,Z,_width);
	//glutSolidSphere(1,16,8);

}

void CRenderer::RenderPlane(float _width, float _depth, float _res, unsigned int _texture)
{
	float quadsideWidth = _width / _res;
	float quadsideDepth = _depth / _res;
	_res *= 0.5f;
	
	float x, z;
	glNormal3f(0,1,0);
	glPushMatrix();
	glTranslatef(-_width*0.5f, 0, -_depth*0.5f);
	glBegin( GL_QUADS );
	{
		for(x = 0; x < _width; x += quadsideWidth)
		{
			for(z = 0; z < _depth; z += quadsideDepth)
			{
				glVertex3f( x+quadsideWidth,	0,		z+quadsideDepth );				
				glVertex3f( x+quadsideWidth,	0,		z );				
				glVertex3f( x,					0,		z );				
				glVertex3f( x,					0,		z+quadsideDepth );				
			}
		}				
	}
	glEnd();
	glPopMatrix();


/*
	_width  = _width * 0.5f;
	_depth = _depth* 0.5f;
	
	glNormal3f(0,1,0);
	glBegin( GL_QUADS );{
		glTexCoord2f( 1.0f, 0.0f );
		glVertex3f( _width, 0, _depth );
		
		glTexCoord2f( 1.0f, 1.0f );
		glVertex3f( _width, 0, -_depth );
		
		glTexCoord2f( 0.0f, 1.0f );
		glVertex3f( -_width, 0, -_depth );
		
		glTexCoord2f( 0.0f, 0.0f );
		glVertex3f( -_width, 0, _depth );				
	}glEnd();

/*
	float x, y;
	float tx,ty;

	tx = ty = 1.0f / _res;

	glBindTexture( GL_TEXTURE_2D, _texture );
	
	_width  = _width * 0.5f;
	_depth = _depth* 0.5f;
	_res	= _res	 * 0.5f;
	
	glPushMatrix();
//	glRotatef(90, 1,0,0);
	for(x=-_width; x<_width; x+=_res*2.0f){
		for(y=-_depth; y<_depth; y+=_res*2.0f){
			
			glBegin( GL_QUADS );{
				glNormal3f(0,0,-1);
				
				//glTexCoord2f( 1.0f, 0.0f );
				glTexCoord2f( 1.0f, 0.0f );
				glVertex3f( x+_res, y+_res, 0 );

				glTexCoord2f( 1.0f, 1.0f );
				glVertex3f( x+_res, y-_res, 0 );

				glTexCoord2f( 0.0f, 1.0f );
				glVertex3f( x-_res, y-_res, 0 );

				glTexCoord2f( 0.0f, 0.0f );
				glVertex3f( x-_res, y+_res, 0 );				
			}glEnd();
		}
	}
	glPopMatrix();	
	*/
}

void CRenderer::RenderCube(float _scale, unsigned int _texture)
{
	glBindTexture( GL_TEXTURE_2D, _texture );		

	glPushMatrix();

	//NEHE's CUBE, huzzah!
	glBegin(GL_QUADS);
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-_scale, -_scale,  _scale);	// Point 1 (Front)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( _scale, -_scale,  _scale);	// Point 2 (Front)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( _scale,  _scale,  _scale);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-_scale,  _scale,  _scale);	// Point 4 (Front)
		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-_scale, -_scale, -_scale);	// Point 1 (Back)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-_scale,  _scale, -_scale);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( _scale,  _scale, -_scale);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( _scale, -_scale, -_scale);	// Point 4 (Back)
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-_scale,  _scale, -_scale);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-_scale,  _scale,  _scale);	// Point 2 (Top)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( _scale,  _scale,  _scale);	// Point 3 (Top)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( _scale,  _scale, -_scale);	// Point 4 (Top)
		// Bottom Face
		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-_scale, -_scale, -_scale);	// Point 1 (Bottom)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( _scale, -_scale, -_scale);	// Point 2 (Bottom)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( _scale, -_scale,  _scale);	// Point 3 (Bottom)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-_scale, -_scale,  _scale);	// Point 4 (Bottom)
		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( _scale, -_scale, -_scale);	// Point 1 (Right)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( _scale,  _scale, -_scale);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( _scale,  _scale,  _scale);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( _scale, -_scale,  _scale);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-_scale, -_scale, -_scale);	// Point 1 (Left)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-_scale, -_scale,  _scale);	// Point 2 (Left)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-_scale,  _scale,  _scale);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-_scale,  _scale, -_scale);	// Point 4 (Left)
	glEnd();

	glPopMatrix();


}


void CRenderer::RenderCuboid(vec3f &_scale, unsigned int _texture)
{
	glPushMatrix();
	glBindTexture( GL_TEXTURE_2D, _texture );		
	
	vec3f tscale = _scale * 0.5f;
	//NEHE's CUBE, huzzah!
	glBegin(GL_QUADS);
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-tscale.x, -tscale.y,  tscale.z);	// Point 1 (Front)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( tscale.x, -tscale.y,  tscale.z);	// Point 2 (Front)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( tscale.x,  tscale.y,  tscale.z);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-tscale.x,  tscale.y,  tscale.z);	// Point 4 (Front)
		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-tscale.x, -tscale.y, -tscale.z);	// Point 1 (Back)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-tscale.x,  tscale.y, -tscale.z);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( tscale.x,  tscale.y, -tscale.z);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( tscale.x, -tscale.y, -tscale.z);	// Point 4 (Back)
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-tscale.x,  tscale.y, -tscale.z);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-tscale.x,  tscale.y,  tscale.z);	// Point 2 (Top)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( tscale.x,  tscale.y,  tscale.z);	// Point 3 (Top)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( tscale.x,  tscale.y, -tscale.z);	// Point 4 (Top)
		// Bottom Face
		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-tscale.x, -tscale.y, -tscale.z);	// Point 1 (Bottom)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( tscale.x, -tscale.y, -tscale.z);	// Point 2 (Bottom)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( tscale.x, -tscale.y,  tscale.z);	// Point 3 (Bottom)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-tscale.x, -tscale.y,  tscale.z);	// Point 4 (Bottom)
		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( tscale.x, -tscale.y, -tscale.z);	// Point 1 (Right)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( tscale.x,  tscale.y, -tscale.z);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( tscale.x,  tscale.y,  tscale.z);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( tscale.x, -tscale.y,  tscale.z);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-tscale.x, -tscale.y, -tscale.z);	// Point 1 (Left)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-tscale.x, -tscale.y,  tscale.z);	// Point 2 (Left)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-tscale.x,  tscale.y,  tscale.z);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-tscale.x,  tscale.y, -tscale.z);	// Point 4 (Left)
	glEnd();

	glPopMatrix();	
}